--[[Copyright (C) 2008 by Fritos]]
	
Dailies = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceComm-2.0", "AceEvent-2.0", "FuBarPlugin-2.0", "AceDB-2.0", "AceHook-2.1")

local Tablet = AceLibrary("Tablet-2.0")
local Dewdrop = AceLibrary("Dewdrop-2.0")
local Crayon = AceLibrary("LibCrayon-3.0")
local Deformat = AceLibrary("Deformat-2.0")
local Sink = AceLibrary("LibSink-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Dailies")
local BZ = AceLibrary("LibBabble-Zone-3.0"):GetLookupTable()
local BF = AceLibrary("LibBabble-Faction-3.0"):GetLookupTable()

Dailies.hasIcon  = "Interface\\Addons\\DailiesQuestTracker\\Icon.tga"
Dailies.hasNoText  = false;
Dailies.title = L["Dailies Quest Tracker"];
Dailies.hideMenuTitle = true;
Dailies.cannotDetachTooltip = true;
Dailies:RegisterDB("DailiesDB");
Dailies.clickableTooltip = true;

local DailiesCalendar_MONTH_NAMES = {
	FULLDATE_MONTH_JANUARY,
	FULLDATE_MONTH_FEBRUARY,
	FULLDATE_MONTH_MARCH,
	FULLDATE_MONTH_APRIL,
	FULLDATE_MONTH_MAY,
	FULLDATE_MONTH_JUNE,
	FULLDATE_MONTH_JULY,
	FULLDATE_MONTH_AUGUST,
	FULLDATE_MONTH_SEPTEMBER,
	FULLDATE_MONTH_OCTOBER,
	FULLDATE_MONTH_NOVEMBER,
	FULLDATE_MONTH_DECEMBER,
}

local DailiesCalendar_WEEKDAY_NAMES = {
	WEEKDAY_SUNDAY,
	WEEKDAY_MONDAY,
	WEEKDAY_TUESDAY,
	WEEKDAY_WEDNESDAY,
	WEEKDAY_THURSDAY,
	WEEKDAY_FRIDAY,
	WEEKDAY_SATURDAY,
}

-- Set Debugger
local DailiesDebugEnabled = false

-- Checker for quest announcement
local QuestAnnounced = {}

-- Get the users Expansions installed
local XPacLevel = GetAccountExpansionLevel()

-- Set Current Holiday Variables
local HolidayTitle, HolidayHour, HolidayMinute, HolidayCalendarType, HolidaySequenceType, HolidayEventType, HolidayIcon, HolidayModStatus, HolidayInviteStatus

-- Jump the rep
local RepIncreaseValue = {}

-- Reputation Pane, collapsed headers value holder
local CollapsedHeaders = {}

-- Scheduled reset time for dailies (i.e 3 am Server Time)
local DailiesReset = 3

-- Grab Channel List of Users
local GrabbedChannelList = false

-- Random Dailies values
local DailiesKnownNotCompleted = Crayon:Red(L["Not Completed"])
local DailiesKnownCompleted = Crayon:Green(L["Completed"])

-- Format Fontstring Textures
local TEXTURE_LINK_FORMAT = "|T%s:%d:%d:0:%d|t"

-- Expansion Icons
local TBC_ICON = format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\TBC.tga", 16, 16, 0)
local WotLK_ICON = format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\WotLK.tga", 16, 16, 0)
			
-- Holder for multiple character dailies
local UnitsHolder = {}

--Setup Own formatted Tooltips
QuestUnitsTooltip = CreateFrame("GameTooltip", "UnitsTooltip", nil, "GameTooltipTemplate")
QuestRewardsTooltip = CreateFrame("GameTooltip", "RewardsTooltip", nil, "GameTooltipTemplate")

QuestUnitsTooltip:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
tile = true, tileSize = 16, edgeSize = 16, 
insets = { left = 3, right = 3, top = 3, bottom = 3 }});
											
QuestRewardsTooltip:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
tile = true, tileSize = 16, edgeSize = 16, 
insets = { left = 3, right = 3, top = 3, bottom = 3 }});

-- Comm Setup
Dailies.CommPrefix = "DQTComm";
Dailies:SetCommPrefix(Dailies.CommPrefix);
Dailies.OnCommReceive = {}
DailiesRoster = {}
Dailies.UniqueNames = {};
DailiesCommHolder = {}

-- Dialog Pop-up Messages
DialogOffComms = Crayon:Gold(L["You're turning "]) .. Crayon:Red(L["OFF"]) .. Crayon:Gold(L[" communications for "]) .. Crayon:Green(L["Dailies Quest Tracker"]) .. Crayon:Gold(L[".\n\n Clicking "]).. Crayon:Green(L["YES"])
	.. Crayon:Gold(L[" will reload your UI to remove the channel and you will no longer send and/or receive the Random Dailies!\n\nAre you sure you would like to do this?"])
DialogOnComms = Crayon:Gold(L["You're turning "]) .. Crayon:Green(L["ON"]) .. Crayon:Gold(L[" communications for "]) .. Crayon:Green(L["Dailies Quest Tracker"]) .. Crayon:Gold(L[".\n\n Clicking "]).. Crayon:Green(L["YES"])
	.. Crayon:Gold(L[" will reload your UI to set-up the channel, please make sure your "]) .. Crayon:Green(L["CUSTOM"]) .. Crayon:Gold(L[" channels are set before continuing.\n\nAre you sure you would like to do this?"])

-- Dummy Config Table
local config = {
	type = 'group',
	handler = SCT,
	args = {
	}
}
	
-- Reputation Values holder
local RepValuesHolder = {
	["Shattered Sun Offensive"] = {},
	["Netherwing"] = {},
	["Ogri'la"] = {},
	["Sha'tari Skyguard"] = {},
	["The Kalu'ak"] = {},
	["The Oracles"] = {},	
	["Frenzyheart Tribe"] = {},
	["The Wyrmrest Accord"] = {},	
}

-- Reputation Values
local RepExchange = {
	["Unknown"] = 0,
	["Hated"] = 1,
	["Hostile"] = 2,
	["Unfriendly"] = 3,
	["Neutral"] = 4,
	["Friendly"] = 5,
	["Honored"] = 6,
	["Revered"] = 7,
	["Exalted"] = 8,
}

-- Icons for specific classes
local classIcons = {
	DRUID = "Interface\\Icons\\INV_Misc_MonsterClaw_04",
	WARLOCK = "Interface\\Icons\\Spell_Nature_FaerieFire",
	HUNTER = "Interface\\Icons\\INV_Weapon_Bow_07",
	MAGE = "Interface\\Icons\\INV_Staff_13",
	PRIEST = "Interface\\Icons\\INV_Staff_30",
	WARRIOR = "Interface\\Icons\\INV_Sword_27",
	SHAMAN = "Interface\\Icons\\Spell_Nature_BloodLust",
	PALADIN = "Interface\\Addons\\DailiesQuestTracker\\Icons\\Paladin",
	ROGUE = "Interface\\Addons\\DailiesQuestTracker\\Icons\\Rogue",
	DEATHKNIGHT = "Interface\\Icons\\Spell_Deathknight_ClassIcon",
}

-- Expansion Groups
local ExpansionGroups = {
	["TBC"] = {group = "TBC", title = Crayon:Green("The Burning Crusade"), expansionLevel = 1},
	["WotLK"] = {group = "WotLK", title = Crayon:Blue("Wrath of the Lich King"), expansionLevel = 2},
}

-- Daily Groups
local DailiesGroups = {
	--The Burning Crusade
	["BGPvP"] = {XPac = "TBC", icon = [[Interface\Icons\Achievement_BG_DefendXtowers_AV]], group = "BGPvP", title = L["Battleground"] ..": ["..Crayon:Green(L["Random Daily"]).."]", ValueMax = 1, Accepted = 0, Profession = true},
	["Cooking"] = {XPac = "TBC", icon = [[Interface\Icons\INV_Misc_Food_15]], group ="Cooking", title = GetSpellInfo(2550) .. ": ["..Crayon:Green(L["Random Daily"]).."]", ValueMax = 1, Accepted = 0, Profession = false},
	["DInstance"] = {XPac = "TBC", icon = [[Interface\Icons\Achievement_BG_killingblow_30]], group ="DInstance", title = L["Dungeon Instance"] ..": [".. Crayon:Green(L["Random Daily"]).."]", ValueMax = 1, Accepted = 0, Profession = true},
	["Fishing"] = {XPac = "TBC", icon = [[Interface\Icons\Trade_Fishing]], group ="Fishing", title = GetSpellInfo(7620) .. ": ["..Crayon:Green(L["Random Daily"]).."]", ValueMax = 1, Accepted = 0, Profession = false},
	["Holiday"] = {XPac = "TBC", group ="Holiday", title = L["Holiday"], ValueMax = 1, Accepted = 0, Profession = true}, -- Icon will be made when the Holiday occurs
	["HInstance"] = {XPac = "TBC", icon = [[Interface\Icons\Achievement_BG_killingblow_most]], group ="HInstance", title = L["Heroic Instance"] .. ": [".. Crayon:Green(L["Random Daily"]).."]", ValueMax = 1, Accepted = 0, Profession = true},
	["Netherwing"] = {XPac = "TBC", icon = [[Interface\Addons\DailiesQuestTracker\Icons\Rep\Netherwing.tga]], group ="Netherwing", title = BF["Netherwing"], ValueMax = 8, Accepted = 0, Profession = true},
	["Ogri'la"] = {XPac = "TBC", icon = [[Interface\Addons\DailiesQuestTracker\Icons\Rep\Ogri'la.tga]], group ="Ogri'la", title = BF["Ogri'la"], ValueMax = 2, Accepted = 0, Profession = true},
	["Sha'tari Skyguard"] = {XPac = "TBC", icon = [[Interface\Addons\DailiesQuestTracker\Icons\Rep\Sha'tari Skyguard.tga]], group ="Sha'tari Skyguard", title = BF["Sha'tari Skyguard"], ValueMax = 4, Accepted = 0, Profession = true},
	["Shattered Sun Offensive"] = {XPac = "TBC", icon = [[Interface\Addons\DailiesQuestTracker\Icons\Rep\Shattered Sun Offensive.tga]], group = "Shattered Sun Offensive", title = BF["Shattered Sun Offensive"], ValueMax = 17, Accepted = 0, Profession = true},
	["World PvP"] = {XPac = "TBC", icon = [[Interface\Icons\Achievement_BG_ABshutout]], group ="World PvP", title = L["World PvP"], ValueMax = 2, Accepted = 0, Profession = true},
	
	-- The Wrath of the Lich King
	["BGPvPWotLK"] = {XPac = "WotLK", icon = [[Interface\Icons\Achievement_BG_DefendXtowers_AV]], group = "BGPvPWotLK", title = L["Battleground"] ..": ["..Crayon:Green(L["Random Daily"]).."]", ValueMax = 1, Accepted = 0, Profession = true},
	["Northrend"] = {XPac = "WotLK",  icon = [[Interface\Icons\Achievement_Zone_Northrend_01]], group ="Northrend", title = "Northrend", ValueMax = 2, Accepted = 0, Profession = true},
	["CookingWotLK"] = {XPac = "WotLK", icon = [[Interface\Icons\INV_Misc_Food_15]], group ="CookingWotLK", title = GetSpellInfo(2550) .. ": ["..Crayon:Green(L["Random Daily"]).."]", ValueMax = 1, Accepted = 0, Profession = false},
	["The Kalu'ak"] = {XPac = "WotLK", icon = [[Interface\Addons\DailiesQuestTracker\Icons\Rep\The Kalu'ak.tga]], group ="The Kalu'ak", title = "The Kalu'ak", ValueMax = 3, Accepted = 0, Profession = true},
	["The Oracles"] = {XPac = "WotLK", icon = [[Interface\Addons\DailiesQuestTracker\Icons\Rep\The Oracles.tga]], group ="The Oracles", title = "The Oracles", ValueMax = 8, Accepted = 0, Profession = true},
	["Frenzyheart Tribe"] = {XPac = "WotLK", icon = [[Interface\Addons\DailiesQuestTracker\Icons\Rep\Frenzyheart Tribe.tga]], group ="Frenzyheart Tribe", title = "Frenzyheart Tribe", ValueMax = 8, Accepted = 0, Profession = true},
	["The Wyrmrest Accord"] = {XPac = "WotLK", icon = [[Interface\Addons\DailiesQuestTracker\Icons\Rep\The Wyrmrest Accord.tga]], group ="The Wyrmrest Accord", title = "The Wyrmrest Accord", ValueMax = 1, Accepted = 0, Profession = true},
	["DInstanceWotLK"] = {XPac = "WotLK", icon = [[Interface\Icons\Achievement_BG_killingblow_30]], group ="DInstanceWotLK", title = L["Dungeon Instance"] ..": [".. Crayon:Green(L["Random Daily"]).."]", ValueMax = 1, Accepted = 0, Profession = true},
	["HInstanceWotLK"] = {XPac = "WotLK", icon = [[Interface\Icons\Achievement_BG_killingblow_most]], group ="HInstanceWotLK", title = L["Heroic Instance"] ..": [".. Crayon:Green(L["Random Daily"]).."]", ValueMax = 1, Accepted = 0, Profession = true},
	["WorldPvPWotLK"] = {XPac = "WotLK", icon = [[Interface\Icons\Achievement_BG_ABshutout]], group ="WorldPvPWotLK", title = L["World PvP"], ValueMax = 10, Accepted = 0, Profession = true},

	["Jewelcrafting"] = {XPac = "WotLK", icon = [[Interface\Icons\INV_Misc_Gem_01]], group = "Jewelcrafting", title = GetSpellInfo(25229), ValueMax = 1, Accepted = 0, Profession = false},	
}
	
-- Reputation Table holder
local RepFactionsHolder = {
	["None"] = 4,
	["BGPvP"] = 4,
	["Cooking"] = 4,
	["CookingWotLK"] = 4,
	["Fishing"] = 4,
	["Holiday"] = 4,
	["HInstance"] = 4,
	["HInstanceWotLK"] = 4,	
	["DInstance"] = 4,
	["DInstanceWotLK"] = 4,
	["World PvP"] = 4,
	["WorldPvPWotLK"] = 4,
	["Northrend"] = 4,
	["Shattered Sun Offensive"] = 0,
	["Netherwing"] = 0,
	["Ogri'la"] = 0,
	["Sha'tari Skyguard"] = 0,
	["The Kalu'ak"] = 0,
	["The Oracles"] = 0,
	["Frenzyheart Tribe"] = 0,
	["The Wyrmrest Accord"] = 0,
	["Jewelcrafting"] = 4,
}

 -- Coverter from RGB to HEX
function RGBPercToHex(r, g, b)
	r = r <= 1 and r >= 0 and r or 0
	g = g <= 1 and g >= 0 and g or 0
	b = b <= 1 and b >= 0 and b or 0
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end -- function RGBPercToHex
 
 -- Output Debug Messages to Chatframe
function Dailies:Debug(...)
	if DailiesDebugEnabled == true then
		ChatFrame3:AddMessage(...)
	end
end -- function Debug

-- Resets the Comms to on/off from USERS choice
function Dailies:ResetAllComms()
	ReloadUI();
end -- function ResetAllComms

function Dailies:OnInitialize()
englishFaction = UnitFactionGroup("Player")
PlayerRace = UnitRace("Player");

-- Register Events
Dailies:RegisterEvent("ADDON_LOADED", "ADDON_LOADED")
Dailies:RegisterEvent("CHAT_MSG_SKILL", "CHAT_MSG_SKILL")
Dailies:RegisterEvent("CHAT_MSG_SYSTEM", "DailiesQuest_Tracker")
Dailies:RegisterEvent("UNIT_QUEST_LOG_CHANGED", "DailiesUpdate")
Dailies:RegisterEvent("PLAYER_ALIVE", "PLAYER_ALIVE")
Dailies:RegisterEvent("QUEST_LOG_UPDATE", "QuestObjectives")
Dailies:RegisterEvent("ZONE_CHANGED", "ZONE_CHANGED")
Dailies:RegisterEvent("ZONE_CHANGED_INDOORS", "ZONE_CHANGED")
Dailies:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZONE_CHANGED")
Dailies:RegisterEvent("GOSSIP_SHOW", "GOSSIP_SHOW")
Dailies:RegisterEvent("QUEST_DETAIL", "QUEST_DETAIL")
Dailies:RegisterEvent("VARIABLES_LOADED", "VARIABLES_LOADED")
Dailies:RegisterEvent("PLAYER_ENTERING_WORLD")

Dailies:RegisterBucketEvent("UPDATE_FACTION", 5, "UpdateTheReputation")

Dailies:RegisterEvent("QuestAccepted", "QuestAccepted");
Dailies:RegisterEvent("TurnOnComms", "TurnOnComms");
Dailies:RegisterEvent("CHANNEL_COUNT_UPDATE", "GetChannelMembers");
Dailies:RegisterEvent("SendTheDailies", "SendTheDailies")
	
-- Turn Comm Channels On/Off Dialog
StaticPopupDialogs["RESET_COMMS"] = {
	text = "%s",
	button1 = L["YES"],
	button2 = L["NO"],
	OnAccept = function()
		self.db.char.DailyCommSet  = not self.db.char.DailyCommSet;		
		Dailies:ResetAllComms();
	end,
	OnCancel = function (_, reason)
		if reason == "clicked" then
			SetDailyCommSet = self.db.char.DailyCommSet;
		end
	end,
	timeout = 0,
	whileDead = 1,
	showAlert = 1
};

Dailies:RegisterChatCommand({"/DQT", "/Dailies"}, config)

if englishFaction == "Alliance" then
	-- Battlegrounds
	DailyQuestsList[L["Call to Arms: Warsong Gulch"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Misc_Rune_07", link = "|cffffff00|Hquest:11338:-1|h[Call to Arms: Warsong Gulch]|h|r", coords ="67:34", qgiver = "Alliance Brigadier General", qzone = "Shattrath City", qszone = "Terrace Of Light", area = BZ["Warsong Gulch"], group = "BGPvP", SID = "None:0", ["honor"] = 420, gold = 119900}
	DailyQuestsList[L["Call to Arms: Arathi Basin"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Jewelry_Amulet_07", link = "|cffffff00|Hquest:11335:-1|h[Call to Arms: Arathi Basin]|h|r", coords ="67:34", qgiver = "Alliance Brigadier General", qzone = "Shattrath City", qszone = "Terrace Of Light", area = BZ["Arathi Basin"], group = "BGPvP", SID = "None:0", ["honor"] = 420, gold = 119900}
	DailyQuestsList[L["Call to Arms: Eye of the Storm"]] = {XPac = "TBC", icon = "Interface\\Icons\\Spell_Nature_EyeOfTheStorm", link = "|cffffff00|Hquest:11337:-1|h[Call to Arms: Eye of the Storm]|h|r", coords ="67:34", qgiver = "Alliance Brigadier General", qzone = "Shattrath City", qszone = "Terrace Of Light", area = BZ["Eye of the Storm"], group = "BGPvP", SID = "None:0", ["honor"] = 420, gold = 119900}
	DailyQuestsList[L["Call to Arms: Alterac Valley"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Jewelry_Necklace_21", link = "|cffffff00|Hquest:11336:-1|h[Call to Arms: Alterac Valley]|h|r", coords ="67:34", qgiver = "Alliance Brigadier General", qzone = "Shattrath City", qszone = "Terrace Of Light", area = BZ["Alterac Valley"], group = "BGPvP", SID = "None:0", ["honor"] = 420, gold = 119900}
	-- Regular Instance
	DailyQuestsList[L["Wanted: Shattered Hand Centurions"]] = {XPac = "TBC", icon = "Interface\\Addons\\DailiesQuestTracker\\Icons\\Centurion.tga", link = "|cffffff00|Hquest:11364:70|h[Wanted: Shattered Hand Centurions]|h|r", coords ="75:37", qgiver = "Nether-Stalker Mah'dunn", qzone = "Shattrath City", qszone = "Lower City", loc = BZ["The Shattered Halls"], area = BZ["The Shattered Halls"], group = "DInstance", SID = "None:0", ["rep"] = {BF["Honor Hold"]..":250", BF["The Consortium"]..":250"}, ["rewards"] = {"29460:1"}, gold = 163900} --Shattered Halls
	-- Heroic Instance
	DailyQuestsList[L["Wanted: Nazan's Riding Crop"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Misc_Crop_01", link = "|cffffff00|Hquest:11354:70|h[Wanted: Nazan's Riding Crop]|h|r", coords ="75:37", qgiver = "Wind Trader Zhareem", qzone = "Shattrath City", qszone = "Lower City", loc = BZ["Hellfire Ramparts"], area = BZ["Hellfire Ramparts"], group = "HInstance", SID = "Honor Hold:6", ["rep"] = {BF["Honor Hold"]..":350", BF["The Consortium"]..":350"}, ["rewards"] = {"29434:2"}, gold = 246000} --Ramparts
	DailyQuestsList[L["Wanted: Keli'dan's Feathered Stave"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Staff_Goldfeathered_01", link = "|cffffff00|Hquest:11354:70|h[Wanted: Nazan's Riding Crop]|h|r", coords ="75:37", qgiver = "Wind Trader Zhareem", qzone = "Shattrath City", qszone = "Lower City", loc = BZ["The Blood Furnace"], area = BZ["The Blood Furnace"], group = "HInstance", SID = "Honor Hold:6", ["rep"] = {BF["Honor Hold"]..":350", BF["The Consortium"]..":350"}, ["rewards"] = {"29434:2"}, gold = 246000} --Blood Furnace
	DailyQuestsList[L["Wanted: Bladefist's Seal"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_SummerFest_Symbol_Medium", link = "|cffffff00|Hquest:11363:70|h[Wanted: Bladefist's Seal]|h|r", coords ="75:37", qgiver = "Wind Trader Zhareem", qzone = "Shattrath City", qszone = "Lower City", loc = BZ["The Shattered Halls"], area = BZ["The Shattered Halls"], group = "HInstance", SID = "Honor Hold:6", ["rep"] = {BF["Honor Hold"]..":350", BF["The Consortium"]..":350"}, ["rewards"] = {"29434:2"}, gold = 246000} --Shattered Halls
	--Fishing [Patch 2.4]
	DailyQuestsList[L["Crocolisks in the City"]] = {XPac = "TBC", icon = "Interface\\Icons\\Ability_Hunter_Pet_Crocolisk", link = "|cffffff00|Hquest:11665:70|h[Crocolisks in the City]|h|r", coords ="39:13", qgiver = "Old Man Barlo", qzone = "Terokkar Forest", qszone = "Silmyr Lake", area = BZ["Stormwind City"], group = "Fishing", SID = "None:0", ["rewards"] = {"35348:1"}}
	-- Alliance Nagrand PvP [Patch 2.4]
	DailyQuestsList[L["Spirits of Auchindoun"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Jewelry_FrostwolfTrinket_04", link = "|cffffff00|Hquest:11505:-1|h[Spirits of Auchindoun]|h|r", coords ="56:54", qgiver = "Exorcist Sullivan", qzone = "Terokkar Forest", qszone = "Allerian Stronghold", group = "World PvP", loc = BZ["Terokkar Forest"], area = BZ["Terokkar Forest"], SID = "None:0", ["honor"] = 210, gold = 119900}
	-- Alliance Terokkar PvP [Patch 2.4]
	DailyQuestsList[L["In Defense of Halaa"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Misc_Rune_08", link = "|cffffff00|Hquest:11502:70|h[In Defense of Halaa]|h|r", coords ="56:74", qgiver = "Lakoor", qzone = "Nagrand", qszone = "Telaar", group = "World PvP", loc = BZ["Nagrand"], area = BZ["Nagrand"], SID = "None:0", ["honor"] = 210, gold = 119900}
	BGHonor = "Interface\\Addons\\DailiesQuestTracker\\Icons\\Alliance.tga"
	
elseif englishFaction == "Horde" then
	-- Battlegrounds
	DailyQuestsList[L["Call to Arms: Warsong Gulch"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Misc_Rune_07", link = "|cffffff00|Hquest:11342:-1|h[Call to Arms: Warsong Gulch]|h|r", coords ="66.9:54.4", qgiver = "Horde Warbringer", qzone = "Shattrath City", qszone = "Terrace Of Light", area = BZ["Warsong Gulch"], group = "BGPvP", SID = "None:0", ["honor"] = 420, gold = 119900}
	DailyQuestsList[L["Call to Arms: Arathi Basin"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Jewelry_Amulet_07", link = "|cffffff00|Hquest:11339:-1|h[Call to Arms: Arathi Basin]|h|r", coords ="66.9:54.4", qgiver = "Horde Warbringer", qzone = "Shattrath City", qszone = "Terrace Of Light", area = BZ["Arathi Basin"], group = "BGPvP", SID = "None:0", ["honor"] = 420, gold = 119900}
	DailyQuestsList[L["Call to Arms: Eye of the Storm"]] = {XPac = "TBC", icon = "Interface\\Icons\\Spell_Nature_EyeOfTheStorm", link = "|cffffff00|Hquest:11341:-1|h[Call to Arms: Eye of the Storm]|h|r", coords ="66.9:54.4", qgiver = "Horde Warbringer", qzone = "Shattrath City", qszone = "Terrace Of Light", area = BZ["Eye of the Storm"], group = "BGPvP", SID = "None:0", ["honor"] = 420, gold = 119900}
	DailyQuestsList[L["Call to Arms: Alterac Valley"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Jewelry_Necklace_21", link = "|cffffff00|Hquest:11340:-1|h[Call to Arms: Alterac Valley]|h|r", coords ="66.9:54.4", qgiver = "Horde Warbringer", qzone = "Shattrath City", qszone = "Terrace Of Light", area = BZ["Alterac Valley"], group = "BGPvP", SID = "None:0", ["honor"] = 420, gold = 119900}
	-- Regular Instance
	DailyQuestsList[L["Wanted: Shattered Hand Centurions"]] = {XPac = "TBC", icon = "Interface\\Addons\\DailiesQuestTracker\\Icons\\Centurion.tga", link = "|cffffff00|Hquest:11364:70|h[Wanted: Shattered Hand Centurions]|h|r", coords ="75:37", qgiver = "Wind Trader Zhareem", qzone = "Shattrath City", qszone = "Lower City", loc = BZ["The Shattered Halls"], area = BZ["The Shattered Halls"], group = "DInstance", SID = "None:0", ["rep"] = {BF["Thrallmar"]..":250", BF["The Consortium"]..":250"}, ["rewards"] = {"29460:1"}, gold = 163900} --Shattered Halls
	-- Heroic Instance
	DailyQuestsList[L["Wanted: Nazan's Riding Crop"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Misc_Crop_01", link = "|cffffff00|Hquest:11354:70|h[Wanted: Nazan's Riding Crop]|h|r", coords ="75:37", qgiver = "Wind Trader Zhareem", qzone = "Shattrath City", qszone = "Lower City", loc = BZ["Hellfire Ramparts"], area = BZ["Hellfire Ramparts"], group = "HInstance", SID = "Thrallmar:6", ["rep"] = {BF["Thrallmar"]..":350", BF["The Consortium"]..":350"}, ["rewards"] = {"29434:2"}, gold = 246000} --Ramparts
	DailyQuestsList[L["Wanted: Keli'dan's Feathered Stave"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Staff_Goldfeathered_01", link = "|cffffff00|Hquest:11354:70|h[Wanted: Nazan's Riding Crop]|h|r", coords ="75:37", qgiver = "Wind Trader Zhareem", qzone = "Shattrath City", qszone = "Lower City", loc = BZ["The Blood Furnace"], area = BZ["The Blood Furnace"], group = "HInstance", SID = "Thrallmar:6", ["rep"] = {BF["Thrallmar"]..":350", BF["The Consortium"]..":350"}, ["rewards"] = {"29434:2"}, gold = 246000} --Blood Furnace
	DailyQuestsList[L["Wanted: Bladefist's Seal"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_SummerFest_Symbol_Medium", link = "|cffffff00|Hquest:11363:70|h[Wanted: Bladefist's Seal]|h|r", coords ="75:37", qgiver = "Wind Trader Zhareem", qzone = "Shattrath City", qszone = "Lower City", loc = BZ["The Shattered Halls"], area = BZ["The Shattered Halls"], group = "HInstance", SID = "Thrallmar:6", ["rep"] = {BF["Thrallmar"]..":350", BF["The Consortium"]..":350"}, ["rewards"] = {"29434:2"}, gold = 246000} --Shattered Halls
	--Fishing [Patch 2.4]
	DailyQuestsList[L["Crocolisks in the City"]] = {XPac = "TBC", icon = "Interface\\Icons\\Ability_Hunter_Pet_Crocolisk", link = "|cffffff00|Hquest:11665:70|h[Crocolisks in the City]|h|r", coords ="39:13", qgiver = "Old Man Barlo", qzone = "Terokkar Forest", qszone = "Silmyr Lake", area = BZ["Orgrimmar"], group = "Fishing", SID = "None:0", ["rewards"] = {"35348:1"}}
	-- Horde Nagrand PvP [Patch 2.4]
	DailyQuestsList[L["Spirits of Auchindoun"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Jewelry_FrostwolfTrinket_04", link = "|cffffff00|Hquest:11506:70|h[Spirits of Auchindoun]|h|r", coords ="49.7:46.6", qgiver = "Exorcist Vaisha", qzone = "Terokkar Forest", qszone = "Stonebreaker Hold", group = "World PvP", loc = BZ["Terokkar Forest"], area = BZ["Terokkar Forest"], SID = "None:0", ["honor"] = 210, gold = 119900}
	-- Horde Terokkar PvP [Patch 2.4]
	DailyQuestsList[L["Enemies, Old and New"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Misc_Rune_08", link = "|cffffff00|Hquest:11503:70|h[Enemies, Old and New]|h|r", coords ="49.7:46.6", qgiver = "Karrtog", qzone = "Nagrand", qszone = "Garadar", group = "World PvP", loc = BZ["Nagrand"], area = BZ["Nagrand"], SID = "None:0", ["honor"] = 210, gold = 119900}
	BGHonor = "Interface\\Addons\\DailiesQuestTracker\\Icons\\Horde.tga"
end
	
Dailies:RegisterDefaults('server', {
	SavedDailies = {
		Fishing = nil,
		HInstance = nil,
		DInstance = nil,
		Cooking = nil,
		CookingWotLK = nil,
		BGPvP = nil,
		SavedTime = nil,
	},
	UnitTrackerValid = 0,
	chars = {
	   ['*'] = {
			PlayerClass = nil,
			UnitCanDo = {
			},
			UnitCompleted = {
			},		
		},
	},
})

Dailies:RegisterDefaults('profile', {
	AceCommDelay = 20,
	QuestsVisible = true,
	SinkOptions = {},
	hideDetails = {
		["BGPvP"] = true,
		["Cooking"] = true,
		["Holiday"] = true,
		["Fishing"] = true,
		["HInstance"] = true,
		["HInstanceWotLK"] = true,		
		["DInstance"] = true,
		["DInstanceWotLK"] = true,
		["Netherwing"] = true,
		["Ogri'la"] = true,
		["Sha'tari Skyguard"] = true,
		["Shattered Sun Offensive"] = true,
		["World PvP"] = true,
		["WorldPvPWotLK"] = true,
		["Northrend"] = true,
		["The Kalu'ak"] = true,
		["The Oracles"] = true,
		["Frenzyheart Tribe"] = true,
		["The Wyrmrest Accord"] = true,		
		["TBC"] = true,
		["WotLK"] = true,
		["Jewelcrafting"] = true,
		},
})

Dailies:RegisterDefaults('char', {
	DailyQuestsAccepted = {},
	DailyQuestsCompleted = {},
	DailyQuestsObjectives = {},
	TooltipPlacement = "Right",
	MutilpleCharacters = true,
	TooltipTimer = false,
	TBC = false,
	WotLK = false,
	TimeFormat = "Standard",
	DailyCommSet = false,
	LocationShow = true,
	HideDailyList = false,
	TOMTOM_WAYPOINTS = false,
	CART_WAYPOINTS = false,
	CART_WAYPOINTS_THREE = false,	
	GoldDisplay = 1,
	QuestGiver = true,
	FactionDisplay = "Icon",
	Filters = {
		["BGPvP"] = false,
		["Cooking"] = false,
		["CookingWotLK"] = false,
		["Fishing"] = false,
		["Holiday"] = false,
		["Northrend"] = false,
		["HInstance"] = false,
		["HInstanceWotLK"] = false,		
		["DInstance"] = false,
		["DInstanceWotLK"] = false,
		["Netherwing"] = false,
		["Ogri'la"] = false,
		["Sha'tari Skyguard"] = false,
		["Shattered Sun Offensive"] = false,
		["World PvP"] = false,
		["WorldPvPWotLK"] = false,
		["The Kalu'ak"] = false,
		["The Oracles"] = false,
		["Frenzyheart Tribe"] = false,
		["The Wyrmrest Accord"] = false,
		["Jewelcrafting"] = false,		
	},
	GroupValues = {
		["BGPvP"] = 0,
		["Cooking"] = 0,
		["CookingWotLK"] = 0,
		["Fishing"] = 0,
		["Holiday"] = 0,
		["HInstance"] = 0,
		["HInstanceWotLK"] = 0,		
		["DInstance"] = 0,
		["DInstanceWotLK"] = 0,
		["Netherwing"] = 0,
		["Ogri'la"] = 0,
		["Sha'tari Skyguard"] = 0,
		["Shattered Sun Offensive"] = 0,
		["World PvP"] = 0,
		["WorldPvPWotLK"] = 0,
		["Northrend"] = 0,
		["The Kalu'ak"] = 0,
		["The Oracles"] = 0,
		["Frenzyheart Tribe"] = 0,
		["The Wyrmrest Accord"] = 0,
		["Jewelcrafting"] = 0,		
	}
})

StaticPopupDialogs["RESET_COMMS"].OnShow = function()
	for index = 1, STATICPOPUP_NUMDIALOGS, 1 do
		local frame = getglobal("StaticPopup"..index);
		local alertIcon = getglobal(frame:GetName().."AlertIcon");
		if self.db.char.DailyCommSet == true then
			alertIcon:SetTexture("Interface\\Addons\\DailiesQuestTracker\\Icons\\CommOff.tga")
		else
			alertIcon:SetTexture("Interface\\Addons\\DailiesQuestTracker\\Icons\\CommOn.tga")
		end
	end
end;
	
StaticPopupDialogs["ABANDON_QUEST"].OnAccept = function() Dailies:AbandonedQuest(GetAbandonQuestName()); AbandonQuest(); PlaySound("igQuestLogAbandonQuest"); Dailies:ZONE_CHANGED() end
StaticPopupDialogs["ABANDON_QUEST_WITH_ITEMS"].OnAccept = function() Dailies:AbandonedQuest(GetAbandonQuestName()); AbandonQuest(); PlaySound("igQuestLogAbandonQuest"); Dailies:ZONE_CHANGED() end

Dailies:PLAYER_ALIVE()
LibStub("LibSink-2.0"):Embed(self)
Dailies:SetSinkStorage(self.db.profile.SinkOptions)
end -- function OnInitialize

-- Check dates to add in Seasonal Dailies
function Dailies:DailiesSeasonal(HolidayEvent)
	if HolidayEvent then
		Dailies:Debug("Current Holiday is: " .. HolidayEvent)
		-- Midsummer Fire Festival
		if HolidayEvent == "Midsummer Fire Festival" then
				HolidayIcon = format(TEXTURE_LINK_FORMAT, "Interface\\Icons\\INV_Misc_TabardSummer02", Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0)
				DailiesGroups["Holiday"].title = "Holiday: " .. "[" .. HolidayIcon .. Crayon:Orange(HolidayEvent) .. "]"		
		
				DailiesGroups["Holiday"]["ValueMax"] = 4
				DailyQuestsList[L["More Torch Tossing"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Torch_Lit", link = "|cffffff00|Hquest:11921:-1|h[More Torch Tossing]|h|r", coords ="0:0", qgiver = "Master Fire Eater", qzone = "Capital Cities", qszone = "Nil", group = "Holiday", area = "Capital Cities", SID = "Holiday:0", dates = {starting = "6/21", ending = "7/05"}, ["rewards"] = {"23247:5"},  gold = 119900}
				DailyQuestsList[L["More Torch Catching"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_TradeskillItem_03", link = "|cffffff00|Hquest:11924:-1|h[More Torch Catching]|h|r", coords ="0:0", qgiver = "Master Fire Eater", qzone = "Capital Cities", qszone = "Nil", group = "Holiday", area = "Capital Cities", SID = "Holiday:0", dates = {starting = "6/21", ending = "7/05"}, ["rewards"] = {"23247:5"},  gold = 119900}
				DailyQuestsList[L["Striking Back"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Misc_Gem_AzureDraenite_02", link = "|cff40c040|Hquest:11954:67|h[Striking Back]|h|r", group = "Holiday", coords ="0:0", qgiver = "Earthen Ring Elder", qzone = "Capital Cities", qszone = "Nil", area = BZ["Hellfire Peninsula"], SID = "Holiday:0", dates = {starting = "6/21", ending = "7/05"}, ["rewards"] = {"23247:10"},  gold = 79300}
				DailyQuestsList[L["Summon Ahune"]] = {XPac = "TBC", icon = "Interface\\Icons\\Spell_Frost_IceShard", link = "|cff40c040|Hquest:11691:70|h[Summon Ahune]|h|r", coords ="0:0", qgiver = "Luma Skymother", qzone = "The Slave Pens", qszone = "Nil", group = "Holiday", area = BZ["The Slave Pens"], SID = "Holiday:0", dates = {starting = "6/21", ending = "7/05"}, gold = 119900}
			
		-- Brewfest
		elseif HolidayEvent == "Brewfest" then
			HolidayIcon = format(TEXTURE_LINK_FORMAT, "Interface\\Icons\\INV_Holiday_BrewfestBuff_01", Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0)
			DailiesGroups["Holiday"].title = "Holiday: " .. "[" .. HolidayIcon .. Crayon:Orange(HolidayEvent) .. "]"		
		
			DailiesGroups["Holiday"]["ValueMax"] = 3
			if englishFaction == "Alliance" then
				DailyQuestsList["This One Time, When I Was Drunk..."] = {XPac = "TBC", icon = [[Interface\Icons\INV_Misc_Coin_01]], link = "|cffffff00|Hquest:12020:-1|h[This One Time, When I Was Drunk...]|h|r", coords = "38.718:12.800", qgiver = "Old Man Barlo", qzone = "Dun Morogh", qszone = "Nil", area = "Dun Morogh", group = "Holiday", SID = "Holiday:0", dates = {starting = "9/20", ending = "10/4"}, ["rewards"] = {"37829:10"}}
				DailyQuestsList[L["Bark for the Barleybrews!"]] = {XPac = "TBC", icon = [[Interface\Icons\INV_Belt_05]], link = "|cffffff00|Hquest:11293:-1|h[Bark for the Barleybrews!]|h|r", coords ="49.011:39.815", qgiver = "Becan Barleybrew", qzone = "Dun Morogh", qszone = "Nil", area = "Dun Morogh", group = "Holiday", SID = "Holiday:0", dates = {starting = "9/20", ending = "10/4"}, ["rewards"] = {"37829:15"}}
				DailyQuestsList[L["Bark for the Thunderbrews!"]] = {XPac = "TBC", icon = [[Interface\Icons\INV_Belt_05]], link = "|cffffff00|Hquest:11294:-1|h[Bark for the Thunderbrews!]|h|r", coords ="49.457:38.669", qgiver = "Daran Thunderbrew", qzone = "Dun Morogh", qszone = "Nil", area = "Dun Morogh", group = "Holiday", SID = "Holiday:0", dates = {starting = "9/20", ending = "10/4"}, ["rewards"] = {"37829:15"}}	
				DailyQuestsList[L["Insult Coren Direbrew"]] = {XPac = "TBC", icon = [[Interface\Icons\INV_Cask_02]], link = "|cffffff00|Hquest:12062:70|h[Insult Coren Direbrew]|h|r", coords = "0:0", qgiver = "Brewfest Spy", qzone = "Blackrock Depths", qszone = "The Grim Guzzler", area = "Blackrock Depths", group = "Holiday", SID = "Holiday:0", dates = {starting = "9/20", ending = "10/4"}, ["rewards"] = {"37829:10"}}
			
			elseif englishFaction == "Horde" then
				DailyQuestsList[L["Bark for Drohn's Distillery!"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Belt_05", group = "Holiday", SID = "Holiday:0", dates = {starting = "10/02", ending = "10/16"}, ["rewards"] = {"33455:40"}}
				DailyQuestsList[L["Bark for T'chali's Voodoo Brewery!"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Belt_05", group = "Holiday", SID = "Holiday:0", dates = {starting = "10/02", ending = "10/16"}, ["rewards"] = {"33455:40"}}
			end
			
		-- Headless Horseman (Hallow's End)
		elseif HolidayEvent == "Hallow's End" then	
			HolidayIcon = format(TEXTURE_LINK_FORMAT, "Interface\\Icons\\INV_Misc_Bag_28_Halloween", Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0)
			DailiesGroups["Holiday"].title = "Holiday: " .. "[" .. HolidayIcon .. Crayon:Orange(HolidayEvent) .. "]"	
		
			DailiesGroups["Holiday"]["ValueMax"] = 3
			DailyQuestsList["Call the Headless Horseman"] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Misc_Candle_01", link = "|cffffff00|Hquest:11392:70|h[Call the Headless Horseman]|h|r", coords = "0:0", qgiver = "Pumpkin Shrine", qzone = "Scarlet Monestary", qszone = "Graveyard", area = "Scarlet Monestary", group = "Holiday", SID = "Holiday:0", dates = {starting = "10/18", ending = "11/02"},  gold = 7500}
			DailyQuestsList["Stop the Fires!"] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Drink_19", link = "|cffffff00|Hquest:11131:-1|h[Stop the Fires!]|h|r", coords = "0:0", qgiver = "Costumed Orphan Matron", qzone = "Local Towns", qszone = "Nil", area = "Local Towns", group = "Holiday", SID = "Holiday:0", dates = {starting = "10/18", ending = "11/02"},  ["rewards"] = {"20557:3", "34068:1"}, gold = 119900}
			DailyQuestsList["Let the Fires Come!"] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Drink_19", link = "|cffffff00|Hquest:12135:-1|h[Let the Fires Come!]|h|r", coords = "0:0", qgiver = "Costumed Orphan Matron", qzone = "Local Towns", qszone = "Nil", area = "Local Towns", group = "Holiday", SID = "Holiday:0", dates = {starting = "10/18", ending = "11/02"},  ["rewards"] = {"20557:3", "34068:1"}, gold = 119900}
			DailyQuestsList["Smash the Pumpkin"] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Misc_Bag_28_Halloween", link = "|cffffff00|Hquest:12155:-1|h[Smash the Pumpkin]|h|r", coords = "0:0", qgiver = "Large Jack-o'-Latern", qzone = "Local Towns", qszone = "Nil", area = "Local Towns", group = "Holiday", SID = "Holiday:0", dates = {starting = "10/18", ending = "11/02"},  ["rewards"] = {"34077:1"}, gold = 119900}
		else
			Dailies:Debug("No Holiday Information Found")
			DailiesGroups["Holiday"].Profession = false
		end
	end
end -- function DailiesSeasonal

-- Check to see if USER wants unfolding of groups and unfold when necessary
function Dailies:ZONE_CHANGED()
	if self.db.char.LocationShow == true then
		if (Tablet20Frame) and (Tablet20Frame:IsVisible()) then
			return
		else
		
			local DailiesZoneName = GetRealZoneText(); local DailiesSubZoneName = GetSubZoneText();
			local DailiesInInstance, DailiesInstanceType = IsInInstance(); local DailiesDifficulty = GetInstanceDifficulty()
					
			for GroupName, GroupText in pairsByKeys (DailiesGroups) do
				self.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] = true
			end
			
			for index, value in pairs (Dailies.db.char.DailyQuestsAccepted) do
				if DailyQuestsList[index]["area"] then
					if (DailyQuestsList[index]["area"] == DailiesZoneName) or (DailyQuestsList[index]["qzone"] == DailiesZoneName) then
						self.db.profile.hideDetails[DailyQuestsList[index]["group"]] = false
					end
				end
			end
			
			if DailiesInInstance and DailiesInstanceType == "pvp"  then
				self.db.profile.hideDetails["BGPvP"] = false
			end		
			
			if DailiesInInstance then
				if (DailiesInstanceType == "party") and (DailiesDifficulty == 2) then
					for index, value in pairs (Dailies.db.char.DailyQuestsAccepted) do
						if DailyQuestsList[index]["area"] then
							if DailyQuestsList[index]["area"] == DailiesZoneName then
								self.db.profile.hideDetails["DInstance"] = false
							end
							
							if DailyQuestsList[index]["area"] == DailiesZoneName then
								self.db.profile.hideDetails["HInstance"] = false
							end
						end
					end
				end
			else
				if (DailiesInstanceType == "party") and (DailiesDifficulty == 1) then
					for index, value in pairs (Dailies.db.char.DailyQuestsAccepted) do
						if DailyQuestsList[index]["area"] then
							if DailyQuestsList[index]["area"] == DailiesZoneName then
								self.db.profile.hideDetails["DInstance"] = false
							end
						end
					end
				end
			end
		end
	end
end -- function ZONE_CHANGED

function Dailies:VARIABLES_LOADED()
	DailiesTooltips:BuildTooltips()
end -- function VARIABLES_LOADED

-- Check to see if Addons are loaded
function Dailies:ADDON_LOADED(arg1)	
	if (arg1 == "TomTom") then
		Dailies:Debug("TomTom Plugin loaded!")
		TOMTOM_LOADED = true
	end
	
	if (arg1 == "Cartographer_Waypoints") then
		Dailies:Debug("Cartographer_Waypoints Plugin loaded!")
		CARTOGRAPH_LOADED = true
	end
	
	if (arg1 == "Cartographer3_Waypoints") then
		Dailies:Debug("Cartographer3_Waypoints Plugin loaded!")
		CARTOGRAPH_THREE_LOADED = true
	end	
 end -- function ADDON_LOADED
 
 -- Hack for when someone Abandons a daily, so we can reset it in the FuBar tooltip
function Dailies:AbandonedQuest(AbandonedName)
	for QuestNameHolder, NilHolder in pairs (self.db.char.DailyQuestsAccepted) do
		DailiesGroups[DailyQuestsList[QuestNameHolder]["group"]].Accepted = 0
	end
	
	for index, value in pairs (Dailies.db.char.DailyQuestsAccepted) do
		if index == AbandonedName then
			Dailies.db.char.DailyQuestsAccepted[index] = nil;
			Dailies.db.char.DailyQuestsObjectives[index] = nil;
		end
	end
	Dailies:QuestAccepted()
end -- function AbandonedQuest

-- Gold Sorter
function goldByKeys (t, f)
	local TempHolder = {}
	for n in pairs (t) do 
		table.insert (TempHolder, n) 
	end
	
	table.sort(TempHolder, function(a, b)
		if t[a]["gold"] == t[b]["gold"] then
			return a < b
		else
			return (t[a]["gold"] or 0) > (t[b]["gold"] or 0)
		end
	end)
		
	local i = 0
	return function ()
	i = i + 1
	return TempHolder[i], t[TempHolder[i]]
	end
end -- function goldByKeys

-- Title Sorter
function pairsByKeys (t, f)
	local TempHolder = {}
	for n in pairs (t) do 
		table.insert (TempHolder, n) 
	end
	table.sort (TempHolder, f)
	local i = 0
	return function ()
	i = i + 1
	return TempHolder[i], t[TempHolder[i]]
	end
end -- function pairByKeys

-- Checks to see if the USER has finished all the objectives in a quest
function Dailies:QuestObjectives()
	local numEntries, numQuests = GetNumQuestLogEntries()

	if numEntries > 0 then
		for QuestID = 1, numEntries do
			SelectQuestLogEntry(QuestID)
			local QuestName, _, _, _, QuestHeader, QuestCollapsed, QuestCompleted, QuestIsDaily = GetQuestLogTitle(QuestID)
			if QuestCompleted and QuestIsDaily and (QuestAnnounced[QuestName] ~= true) then
				QuestAnnounced[QuestName] = true
				if Dailies.db.char.DailyQuestsAccepted[QuestName] then
					Dailies:Debug ("Objectives Completed!")
					Dailies.db.char.DailyQuestsObjectives[QuestName] = "Turn-In"
					
					PlaySoundFile([[Sound\Interface\PVPFlagCapturedmono.wav]])
					if DailyQuestsList[QuestName]["coords"] then
						if DailyQuestsList[QuestName].donor then
							self:Pour("Turn in " .. DailyQuestsList[QuestName].link .. " to " .. Crayon:Green(DailyQuestsList[QuestName].donor) .. "\n" .. Crayon:White(DailyQuestsList[QuestName].donorszone) .. Crayon:Silver(", ") .. Crayon:White(DailyQuestsList[QuestName].donorzone), nil, nil, nil, nil, nil, nil, nil, "NOTIFY", DailyQuestsList[QuestName]["icon"])		
						else
							if DailyQuestsList[QuestName]["qszone"] == "Nil" then
								self:Pour("Turn in " .. DailyQuestsList[QuestName].link .. " to " .. Crayon:Green(DailyQuestsList[QuestName]["qgiver"]) .. "\n" .. Crayon:White(DailyQuestsList[QuestName]["qzone"]), nil, nil, nil, nil, nil, nil, nil, "NOTIFY", DailyQuestsList[QuestName]["icon"])
							else
								self:Pour("Turn in " .. DailyQuestsList[QuestName].link .. " to " .. Crayon:Green(DailyQuestsList[QuestName]["qgiver"]) .. "\n" .. Crayon:White(DailyQuestsList[QuestName]["qszone"]) .. Crayon:Silver(", ") .. Crayon:White(DailyQuestsList[QuestName]["qzone"]), nil, nil, nil, nil, nil, nil, nil, "NOTIFY", DailyQuestsList[QuestName]["icon"])		
							end						
						end
					end
				end
			end
		end
	end
end -- function QuestObjectives

-- Check for EVENT triggers
function Dailies:DailiesQuest_Tracker(msg)
	-- /script Dailies:DailiesQuest_Tracker("You can complete 25 daily quests today.")
	-- /script Dailies:DailiesQuest_Tracker("Quest accepted: Super Hot Stew")
	-- /script Dailies:DailiesQuest_Tracker("Smash the Pumpkin completed.")
	-- /script Dailies:DailiesQuest_Tracker("You are now Neutral with Thrallmar.")
	
	_, ChatFrameFontSize = GetChatWindowInfo(1)
	local QuestNameCompleted = Deformat(msg, ERR_QUEST_COMPLETE_S) -- "%s completed."
	local QuestNameAccepted = Deformat(msg, ERR_QUEST_ACCEPTED_S)
	local DailiesQuestReset = Deformat(msg, DAILY_QUESTS_REMAINING)
	local DailiesFactionChanged, DailiesFactionName = Deformat(msg, FACTION_STANDING_CHANGED)
		
	if DailiesQuestReset == GetMaxDailyQuests() then
		Dailies:Debug(L["Dailies Have Reset!"])
		ResetText = Crayon:Green(L["Dailies have reset for the day."])
		DEFAULT_CHAT_FRAME:AddMessage (" |TInterface\\Addons\\DailiesQuestTracker\\Icons\\Reset.tga:" .. ChatFrameFontSize .. "|t" .. ResetText); 
		self:Pour(ResetText, nil, nil, nil, nil, nil, nil, nil, "NOTIFY", "Interface\\Addons\\DailiesQuestTracker\\Icons\\Reset.tga")
		self.db.char.DailyQuestsCompleted = {}; self.db.server.SavedDailies = {}; DailiesCommHolder = {};
		self.db.char.GroupValues = {
			["BGPvP"] = 0,
			["Cooking"] = 0,
			["CookingWotLK"] = 0,
			["Fishing"] = 0,
			["Holiday"] = 0,
			["HInstance"] = 0,
			["HInstanceWotLK"] = 0,			
			["DInstance"] = 0,
			["DInstanceWotLK"] = 0,
			["Netherwing"] = 0,
			["Ogri'la"] = 0,
			["Sha'tari Skyguard"] = 0,
			["Shattered Sun Offensive"] = 0,
			["World PvP"] = 0,
			["WorldPvPWotLK"] = 0,
			["Northrend"] = 0,
			["The Kalu'ak"] = 0,	
			["The Oracles"] = 0,	
			["Frenzyheart Tribe"] = 0,
			["The Wyrmrest Accord"] = 0,
			["Jewelcrafting"] = 0,			
		}

		if Dailies.db.server.chars then
			for UnitName in pairs (Dailies.db.server.chars) do
				for UnitQuest in pairs (Dailies.db.server.chars[UnitName].UnitCompleted) do
					Dailies.db.server.chars[UnitName].UnitCompleted[UnitQuest] = nil
				end
			end
		end
		Dailies:Update()
	end

	
	if DailiesFactionChanged then
		Dailies:ExpandTheReputation(DailiesFactionChanged, DailiesFactionName)
	end
	
	if QuestNameAccepted then
		for QuestAcceptedIndex, QuestAcceptedValue in pairs(DailyQuestsList) do
			if (QuestNameAccepted == QuestAcceptedIndex) then
				DailiesGroups[DailyQuestsList[QuestAcceptedIndex].group].Accepted = DailiesGroups[DailyQuestsList[QuestAcceptedIndex].group].Accepted + 1			
				Dailies:Debug(L["Quest Accepted!"])
				local QuestHour, QuestMinute = GetGameTime(); local TwentyFourTime = QuestHour .. ":" .. QuestMinute; QuestTimeAccepted = TwentyFourTime
				Dailies.db.char.DailyQuestsAccepted[QuestNameAccepted] = QuestTimeAccepted
				
				if self.db.char.TimeFormat == "Standard" then
					local QuestPM = floor(QuestHour / 12) == 1; QuestHour = mod(QuestHour, 12); if QuestHour == 0 then QuestHour = 12 end;
					local QuestTime = string.format("%d" .. Crayon:Gold(":") .. "%02d" .. Crayon:Gold(" %s"), QuestHour, QuestMinute, QuestPM and "PM" or "AM");
					self:Pour((DailyQuestsList[QuestNameAccepted].link or QuestAcceptedIndex) .. L[" accepted at "] .. QuestTime, nil, nil, nil, nil, nil, nil, nil, "NOTIFY", DailyQuestsList[QuestNameAccepted]["icon"])
				else
					local QuestTime = QuestHour .. Crayon:Gold(":") .. QuestMinute
					self:Pour((DailyQuestsList[QuestNameAccepted].link or QuestAcceptedIndex) .. L[" accepted at "] .. QuestTime, nil, nil, nil, nil, nil, nil, nil, "NOTIFY", DailyQuestsList[QuestNameAccepted]["icon"])
				end
				
				QuestAnnounced[QuestNameAccepted] = false
				Dailies.db.server.UnitTrackerValid = 1
				--Dailies:QuestAccepted()
				Dailies:Debug(DailyQuestsList[QuestAcceptedIndex]["group"] .. " accepted value is " .. DailiesGroups[DailyQuestsList[QuestAcceptedIndex]["group"]].Accepted)
				Dailies:ZONE_CHANGED()
			end
		end
	end
	
	if QuestNameCompleted then
		for QuestCompletedIndex, QuestCompletedValue in pairs(DailyQuestsList) do
			if (QuestNameCompleted == QuestCompletedIndex) then
				DailiesGroups[DailyQuestsList[QuestCompletedIndex].group].Accepted = DailiesGroups[DailyQuestsList[QuestCompletedIndex].group].Accepted - 1
				
				Dailies:Debug(L["Quest Completed!"])
				local QuestHour, QuestMinute = GetGameTime(); local TwentyFourTime = QuestHour .. ":" .. QuestMinute; QuestTimeCompleted = TwentyFourTime; local UnitCompleteTime = DailiesNumberDate .. ":" .. TwentyFourTime
				
				if self.db.char.TimeFormat == "Standard" then
					local QuestPM = floor(QuestHour / 12) == 1; QuestHour = mod(QuestHour, 12); if QuestHour == 0 then QuestHour = 12 end;
					local QuestTime = string.format("%d" .. Crayon:Gold(":") .. "%02d" .. Crayon:Gold(" %s"), QuestHour, QuestMinute, QuestPM and "PM" or "AM");
					self:Pour(DailyQuestsList[QuestNameCompleted].link .. L[" completed at "] .. QuestTime, nil, nil, nil, nil, nil, nil, nil, "NOTIFY", DailyQuestsList[QuestNameCompleted]["icon"])
				else
					local QuestTime = QuestHour .. Crayon:Gold(":") .. QuestMinute
					self:Pour(DailyQuestsList[QuestNameCompleted].link .. L[" completed at "] .. QuestTime, nil, nil, nil, nil, nil, nil, nil, "NOTIFY", DailyQuestsList[QuestNameCompleted]["icon"])
				end
				
				Dailies.db.char.DailyQuestsCompleted[QuestNameCompleted] = UnitCompleteTime; Dailies.db.char.DailyQuestsAccepted[QuestNameCompleted] = nil; Dailies.db.char.DailyQuestsObjectives[QuestNameCompleted] = nil; Dailies.db.server.chars[UnitName("player")].UnitCompleted[QuestNameCompleted] = UnitCompleteTime
				--Dailies:QuestAccepted()
				Dailies:QuestValue(QuestNameCompleted)
				Dailies:Debug(DailyQuestsList[QuestNameCompleted]["group"] .. " accepted value is " .. DailiesGroups[DailyQuestsList[QuestNameCompleted]["group"]].Accepted)
				Dailies:ZONE_CHANGED()
			end
		end
	end
end -- function DailiesQuest_Tracker

-- Holder and Counter for Accepted Quests
function Dailies:QuestAccepted()
	if self.db.char.DailyQuestsAccepted then
		for QuestNameHolderNiller in pairs (self.db.char.DailyQuestsAccepted) do
			DailiesGroups[DailyQuestsList[QuestNameHolderNiller]["group"]].Accepted = 0
		end
	end
	
	if self.db.char.DailyQuestsAccepted then
		for QuestNameHolder in pairs (self.db.char.DailyQuestsAccepted) do
			if DailyQuestsList[QuestNameHolder]["group"] == "Cooking" then
				DailiesGroups["Cooking"].Accepted = DailiesGroups["Cooking"].Accepted + 1
				
			elseif DailyQuestsList[QuestNameHolder]["group"] == "Shattered Sun Offensive" then
				DailiesGroups["Shattered Sun Offensive"].Accepted = DailiesGroups["Shattered Sun Offensive"].Accepted + 1
				
			elseif DailyQuestsList[QuestNameHolder]["group"] == "World PvP" then
				DailiesGroups["World PvP"].Accepted = DailiesGroups["World PvP"].Accepted + 1

			elseif DailyQuestsList[QuestNameHolder]["group"] == "WorldPvPWotLK" then
				DailiesGroups["WorldPvPWotLK"].Accepted = DailiesGroups["WorldPvPWotLK"].Accepted + 1				
				
			elseif DailyQuestsList[QuestNameHolder]["group"] == "Fishing" then
				DailiesGroups["Fishing"].Accepted = DailiesGroups["Fishing"].Accepted + 1
				
			elseif DailyQuestsList[QuestNameHolder]["group"] == "Netherwing" then
				DailiesGroups["Netherwing"].Accepted = DailiesGroups["Netherwing"].Accepted + 1
				
			elseif DailyQuestsList[QuestNameHolder]["group"] == "Ogri'la" then
				DailiesGroups["Ogri'la"].Accepted = DailiesGroups["Ogri'la"].Accepted + 1
				
			elseif DailyQuestsList[QuestNameHolder]["group"] == "Sha'tari Skyguard" then
				DailiesGroups["Sha'tari Skyguard"].Accepted = DailiesGroups["Sha'tari Skyguard"].Accepted + 1
				
			elseif DailyQuestsList[QuestNameHolder]["group"] == "Holiday" then
				DailiesGroups["Holiday"].Accepted = DailiesGroups["Holiday"].Accepted + 1

			elseif DailyQuestsList[QuestNameHolder]["group"] == "BGPvP" then
				DailiesGroups["BGPvP"].Accepted = DailiesGroups["BGPvP"].Accepted + 1			

			elseif DailyQuestsList[QuestNameHolder]["group"] == "DInstance" then
				DailiesGroups["DInstance"].Accepted = DailiesGroups["DInstance"].Accepted + 1

			elseif DailyQuestsList[QuestNameHolder]["group"] == "DInstanceWotLK" then
				DailiesGroups["DInstanceWotLK"].Accepted = DailiesGroups["DInstanceWotLK"].Accepted + 1
				
			elseif DailyQuestsList[QuestNameHolder]["group"] == "HInstance" then
				DailiesGroups["HInstance"].Accepted = DailiesGroups["HInstance"].Accepted + 1

			elseif DailyQuestsList[QuestNameHolder]["group"] == "HInstanceWotLK" then
				DailiesGroups["HInstanceWotLK"].Accepted = DailiesGroups["HInstanceWotLK"].Accepted + 1				
				
			elseif DailyQuestsList[QuestNameHolder]["group"] == "Northrend" then
				DailiesGroups["Northrend"].Accepted = DailiesGroups["Northrend"].Accepted + 1

			elseif DailyQuestsList[QuestNameHolder]["group"] == "CookingWotLK" then
				DailiesGroups["CookingWotLK"].Accepted = DailiesGroups["CookingWotLK"].Accepted + 1

			elseif DailyQuestsList[QuestNameHolder]["group"] == "Jewelcrafting" then
				DailiesGroups["Jewelcrafting"].Accepted = DailiesGroups["Jewelcrafting"].Accepted + 1

			elseif DailyQuestsList[QuestNameHolder]["group"] == "The Kalu'ak" then
				DailiesGroups["The Kalu'ak"].Accepted = DailiesGroups["The Kalu'ak"].Accepted + 1

			elseif DailyQuestsList[QuestNameHolder]["group"] == "The Oracles" then
				DailiesGroups["The Oracles"].Accepted = DailiesGroups["The Oracles"].Accepted + 1

			elseif DailyQuestsList[QuestNameHolder]["group"] == "Frenzyheart Tribe" then
				DailiesGroups["Frenzyheart Tribe"].Accepted = DailiesGroups["Frenzyheart Tribe"].Accepted + 1

			elseif DailyQuestsList[QuestNameHolder]["group"] == "The Wyrmrest Accord" then
				DailiesGroups["The Wyrmrest Accord"].Accepted = DailiesGroups["The Wyrmrest Accord"].Accepted + 1
			end
		end
	end
end -- function QuestAccepted

-- Holder and Counter for groups counts
function Dailies:QuestValue(QuestName)
	Dailies:Debug(L["Adding Group Values!"])
	CookingValue = Dailies.db.char.GroupValues["Cooking"]
	if DailyQuestsList[QuestName]["group"] == "Cooking" then CookingValue = CookingValue + 1 end; Dailies.db.char.GroupValues["Cooking"] = CookingValue
	
	WorldPvPValue = Dailies.db.char.GroupValues["World PvP"]
	if DailyQuestsList[QuestName]["group"] == "World PvP" then WorldPvPValue = WorldPvPValue + 1 end; Dailies.db.char.GroupValues["World PvP"] = WorldPvPValue
	
	WorldPvPWotLKValue = Dailies.db.char.GroupValues["WorldPvPWotLK"]
	if DailyQuestsList[QuestName]["group"] == "WorldPvPWotLK" then WorldPvPWotLKValue = WorldPvPWotLKValue + 1 end; Dailies.db.char.GroupValues["WorldPvPWotLK"] = WorldPvPWotLKValue	

	FishingValue = Dailies.db.char.GroupValues["Fishing"]
	if DailyQuestsList[QuestName]["group"] == "Fishing" then FishingValue = FishingValue + 1 end; Dailies.db.char.GroupValues["Fishing"] = FishingValue	
	
	NetherwingValue = Dailies.db.char.GroupValues["Netherwing"]
	if DailyQuestsList[QuestName]["group"] == "Netherwing" then NetherwingValue = NetherwingValue + 1 end; Dailies.db.char.GroupValues["Netherwing"] = NetherwingValue
	
	SSOValue = Dailies.db.char.GroupValues["Shattered Sun Offensive"]
	if DailyQuestsList[QuestName]["group"] == "Shattered Sun Offensive" then SSOValue = SSOValue + 1 end; Dailies.db.char.GroupValues["Shattered Sun Offensive"] = SSOValue

	OgrilaValue = Dailies.db.char.GroupValues["Ogri'la"]
	if DailyQuestsList[QuestName]["group"] == "Ogri'la" then OgrilaValue = OgrilaValue + 1 end; Dailies.db.char.GroupValues["Ogri'la"] = OgrilaValue
		
	SkyguardValue = Dailies.db.char.GroupValues["Sha'tari Skyguard"]
	if DailyQuestsList[QuestName]["group"] == "Sha'tari Skyguard" then SkyguardValue = SkyguardValue + 1 end; Dailies.db.char.GroupValues["Sha'tari Skyguard"] = SkyguardValue
	
	HolidayValue = Dailies.db.char.GroupValues["Holiday"]
	if DailyQuestsList[QuestName]["group"] == "Holiday" then HolidayValue = HolidayValue + 1 end; Dailies.db.char.GroupValues["Holiday"] = HolidayValue	

	BGPvPValue = Dailies.db.char.GroupValues["BGPvP"]
	if DailyQuestsList[QuestName]["group"] == "BGPvP" then BGPvPValue = BGPvPValue + 1 end; Dailies.db.char.GroupValues["BGPvP"] = BGPvPValue	

	DInstanceValue = Dailies.db.char.GroupValues["DInstance"]
	if DailyQuestsList[QuestName]["group"] == "DInstance" then DInstanceValue = DInstanceValue + 1 end; Dailies.db.char.GroupValues["DInstance"] = DInstanceValue	

	DInstanceWotLKValue = Dailies.db.char.GroupValues["DInstanceWotLK"]
	if DailyQuestsList[QuestName]["group"] == "DInstanceWotLK" then DInstanceWotLKValue = DInstanceWotLKValue + 1 end; Dailies.db.char.GroupValues["DInstanceWotLK"] = DInstanceWotLKValue	
	
	HInstanceValue = Dailies.db.char.GroupValues["HInstance"]
	if DailyQuestsList[QuestName]["group"] == "HInstance" then HInstanceValue = HInstanceValue + 1 end; Dailies.db.char.GroupValues["HInstance"] = HInstanceValue	

	HInstanceWotLKValue = Dailies.db.char.GroupValues["HInstanceWotLK"]
	if DailyQuestsList[QuestName]["group"] == "HInstanceWotLK" then HInstanceWotLKValue = HInstanceWotLKValue + 1 end; Dailies.db.char.GroupValues["HInstanceWotLK"] = HInstanceWotLKValue	
	
	NorthrendValue = Dailies.db.char.GroupValues["Northrend"]
	if DailyQuestsList[QuestName]["group"] == "Northrend" then NorthrendValue = NorthrendValue + 1 end; Dailies.db.char.GroupValues["Northrend"] = NorthrendValue
	
	CookingWotLKValue = Dailies.db.char.GroupValues["CookingWotLK"]
	if DailyQuestsList[QuestName]["group"] == "CookingWotLK" then CookingWotLKValue = CookingWotLKValue + 1 end; Dailies.db.char.GroupValues["CookingWotLK"] = CookingWotLKValue		

	KaluakValue = Dailies.db.char.GroupValues["The Kalu'ak"]
	if DailyQuestsList[QuestName]["group"] == "The Kalu'ak" then KaluakValue = KaluakValue + 1 end; Dailies.db.char.GroupValues["The Kalu'ak"] = KaluakValue

	OraclesValue = Dailies.db.char.GroupValues["The Oracles"]
	if DailyQuestsList[QuestName]["group"] == "The Oracles" then OraclesValue = OraclesValue + 1 end; Dailies.db.char.GroupValues["The Oracles"] = OraclesValue

	FrenzyheartValue = Dailies.db.char.GroupValues["Frenzyheart Tribe"]
	if DailyQuestsList[QuestName]["group"] == "Frenzyheart Tribe" then FrenzyheartValue = FrenzyheartValue + 1 end; Dailies.db.char.GroupValues["Frenzyheart Tribe"] = FrenzyheartValue

	WyrmrestValue = Dailies.db.char.GroupValues["The Wyrmrest Accord"]
	if DailyQuestsList[QuestName]["group"] == "The Wyrmrest Accord" then WyrmrestValue = WyrmrestValue + 1 end; Dailies.db.char.GroupValues["The Wyrmrest Accord"] = WyrmrestValue
	
	JewelcraftingValue = Dailies.db.char.GroupValues["Jewelcrafting"]
	if DailyQuestsList[QuestName]["group"] == "Jewelcrafting" then JewelcraftingValue = JewelcraftingValue + 1 end; Dailies.db.char.GroupValues["Jewelcrafting"] = JewelcraftingValue
	
	end -- function QuestValue

function Dailies:DailiesUpdate()
	DQMax = GetMaxDailyQuests();
	DQComplete = GetDailyQuestsCompleted();
	Dailies:Update()
end -- function DailiesUpdate

-- Turns on Comms
function Dailies:TurnOnComms()
	if (self.db.char.DailyCommSet == false) then
		return
	else
	Dailies:RegisterComm(Dailies.CommPrefix, "GLOBAL")
	Dailies:RegisterComm(Dailies.CommPrefix, "WHISPER")
	Dailies:ScheduleRepeatingEvent("SendTheDailies", 15)
	
	_, ChatFrameFontSize, _, _, _, _, _, _, _ = GetChatWindowInfo(1)
	CommIcon = format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\Comm.tga", ChatFrameFontSize, ChatFrameFontSize, 0)
	CommText = Crayon:Green(L[" You have joined the GLOBAL AceComm channel."])
	DEFAULT_CHAT_FRAME:AddMessage (CommIcon .. CommText);		
	end
end -- function TurnOnComms

function Dailies:PLAYER_ENTERING_WORLD()
	local AddonLoaded, Reason = LoadAddOn("Blizzard_Calendar")
	local GameHour, GameMinute = GetGameTime()

	if AddonLoaded and not HolidayTitle then
		Dailies:Debug("Blizzard_Calendar Loaded")
		-- Open the Calendar, so we can grab te Holiday info
		Calendar_Toggle()
		local DailiesWeekDay, DailiesMonth, DailiesDay, DailiesYear = CalendarGetDate()
		--HolidayTitle, HolidayDiscription, HolidayTexture  = CalendarGetHolidayInfo(0, DailiesDay, 1)
		HolidayTitle, HolidayHour, HolidayMinute, HolidayCalendarType, HolidaySequenceType, HolidayEventType, HolidayIcon, HolidayModStatus, HolidayInviteStatus = CalendarGetDayEvent(0, DailiesDay, 1)

		--[[Dailies:Debug("Title: " .. HolidayTitle)		
		Dailies:Debug("Hour: " .. HolidayHour)
		Dailies:Debug("Minute: " .. HolidayMinute)
		Dailies:Debug("Calendar Type: " .. HolidayCalendarType)
		Dailies:Debug("Sequence Type: " .. HolidaySequenceType)
		Dailies:Debug("Event Type: " .. HolidayEventType)
		Dailies:Debug("Texture: " .. HolidayTexture)
		Dailies:Debug("Mod Status: " .. HolidayModStatus)	
		Dailies:Debug("Invite Status: " .. HolidayInviteStatus)]]				
		
			
		if HolidayTitle and HolidayCalendarType == "HOLIDAY" then
			if (GameHour >= HolidayHour) and HolidaySequenceType == "END" then
				DailiesGroups["Holiday"].Profession = false
				Dailies:Debug("No Holiday Information Found")
			else
				Dailies:DailiesSeasonal(HolidayTitle)
			end
		else
			DailiesGroups["Holiday"].Profession = false
			Dailies:Debug("No Holiday Information Found")
		end
		Calendar_Toggle()
	end
end

function Dailies:OnEnable()
	DQComplete = GetDailyQuestsCompleted();
	DQMax = GetMaxDailyQuests();
	
	if DQComplete == 0 then
		self.db.char.DailyQuestsCompleted = {};
		self.db.char.GroupValues = {
			["BGPvP"] = 0,
			["Cooking"] = 0,
			["CookingWotLK"] = 0,
			["Fishing"] = 0,
			["Holiday"] = 0,
			["HInstance"] = 0,
			["HInstanceWotLK"] = 0,			
			["DInstance"] = 0,
			["DInstanceWotLK"] = 0,		
			["Netherwing"] = 0,
			["Ogri'la"] = 0,
			["Sha'tari Skyguard"] = 0,
			["Shattered Sun Offensive"] = 0,
			["World PvP"] = 0,
			["WorldPvPWotLK"] = 0,
			["Northrend"] = 0,
			["The Kalu'ak"] = 0,
			["The Oracles"] = 0,	
			["Frenzyheart Tribe"] = 0,
			["The Wyrmrest Accord"] = 0,
			["Jewelcrafting"] = 0,
		}
	end
	
	hour, minute = GetGameTime();
	DQStartHour = hour;
	DQStartMiniute = minute;
	SetDailyCommSet = self.db.char.DailyCommSet;
	Dailies:ScheduleEvent("TurnOnComms", self.db.profile.AceCommDelay)
	Dailies:ScheduleEvent("QuestAccepted", 5)
end -- function OnEnable

function Dailies:UpdateTheReputation()
	Dailies:ExpandTheReputation()
end -- function UpdateTheReputation

function Dailies:ExpandTheReputation(arg1, arg2)
	if arg1 then
		RepIncreaseValue = RepExchange[arg1]
	end
	
	Dailies:Debug ("Reputation Updated!")
	
	for i = 1, GetNumFactions() do
		FactionName, FactionDescription, StandingID, BottomValue, TopValue, EarnedValue, atWarWith, canToggleAtWar, isHeader, isCollapsed = GetFactionInfo(i)

		RepFactionsHolder[FactionName] = StandingID
		
		if RepValuesHolder[FactionName] then
			RepValuesHolder[FactionName].title = RepFactionColors[StandingID].title
			RepValuesHolder[FactionName].StandingID = StandingID
			RepValuesHolder[FactionName].ReputationIs = (EarnedValue - BottomValue)
			RepValuesHolder[FactionName].ReputationMax = (TopValue - BottomValue)
			RepValuesHolder[FactionName].ReputationAbsolute = EarnedValue
		end
		
		if isHeader == 1 then
			if isCollapsed == 1 then
				local NumFactions = GetNumFactions()
				ExpandFactionHeader(i)
				NumFactions = GetNumFactions() - NumFactions
				for j = i+1, i+NumFactions do
					local FactionName, _, StandingID, BottomValue, TopValue, EarnedValue = GetFactionInfo(j)
			
					if RepValuesHolder[FactionName] then
						RepValuesHolder[FactionName].title = RepFactionColors[StandingID].title
						RepValuesHolder[FactionName].StandingID = StandingID
						RepValuesHolder[FactionName].ReputationIs = (EarnedValue - BottomValue)
						RepValuesHolder[FactionName].ReputationMax = (TopValue - BottomValue)
						RepValuesHolder[FactionName].ReputationAbsolute = EarnedValue
					end
					
					RepFactionsHolder[FactionName] = StandingID		
				end
				CollapseFactionHeader(i)
			end
		end				

		-- The Deadliest Trap Ever Laid (Scryer or Aldor version)
		if (RepFactionsHolder["The Scryers"]) and (RepFactionsHolder["The Scryers"] >= 4) then
				DailyQuestsList[L["The Deadliest Trap Ever Laid"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Misc_StoneTablet_11", link = "|cffffff00|Hquest:11097:70|h[The Deadliest Trap Ever Laid]|h|r", coords ="56:69", qgiver = "Commander Hobb", qzone = "Shadowmoon Valley", qszone = "Sanctum of the Stars", donor = "Overlord Mor'ghor", donorzone = "Shadowmoon Valley", donorszone = "Netherwing Ledge", area = BZ["Shadowmoon Valley"], group = "Netherwing", SID = "Netherwing:7", ["rep"] = {BF["Netherwing"]..":500"}, gold = 182800}
		end

		if (RepFactionsHolder["The Aldor"]) and (RepFactionsHolder["The Aldor"] >= 4) then
				DailyQuestsList[L["The Deadliest Trap Ever Laid"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Misc_StoneTablet_11", link = "|cffffff00|Hquest:11101:70|h[The Deadliest Trap Ever Laid]|h|r", coords ="62.388:29.289", qgiver = "Commander Arcus", qzone = "Shadowmoon Valley", qszone = "Altar of Sha'tar", donor = "Overlord Mor'ghor", donorzone = "Shadowmoon Valley", donorszone = "Netherwing Ledge", area = BZ["Shadowmoon Valley"], group = "Netherwing", SID = "Netherwing:7", ["rep"] = {BF["Netherwing"]..":500"}, gold = 182800}
		end
		
		-- Hellfire World PvP that requires Reputation
		if englishFaction == "Alliance" then
			if (RepFactionsHolder["Honor Hold"]) and (RepFactionsHolder["Honor Hold"] >= 4) then
				DailyQuestsList[L["Hellfire Fortifications"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Misc_Token_HonorHold", link = "|cffffff00|Hquest:10106:60|h[Hellfire Fortifications]|h|r", coords ="56.330:62.846", qgiver = "Warrant Officer Tracy Proudwell", qzone = "Hellfire Peninsula", qszone = "Honor Hold", group = "World PvP", loc = BZ["Hellfire Peninsula"], area = BZ["Hellfire Peninsula"], SID = "Honor Hold:4", ["rep"] = {BF["Honor Hold"]..":150"}, ["rewards"] = {"24579:3"}, gold = 11350}
				DailiesGroups["World PvP"]["ValueMax"] = 3
			end
		elseif englishFaction == "Horde" then
			if (RepFactionsHolder["Thrallmar"]) and (RepFactionsHolder["Thrallmar"] >= 4) then
				DailyQuestsList[L["Hellfire Fortifications"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Misc_Token_Thrallmar", link = "|cffffff00|Hquest:10110:70|h[Hellfire Fortifications]|h|r", coords ="55.9:39.1", qgiver = "Battlecryer Blackeye", qzone = "Hellfire Peninsula", qszone = "Thrallmar", group = "World PvP", loc = BZ["Hellfire Peninsula"], area = BZ["Hellfire Peninsula"], SID = "Thrallmar:4", ["rep"] = {BF["Thrallmar"]..":150"}, ["rewards"] = {"24581:3"}, gold = 11350}		
				DailiesGroups["World PvP"]["ValueMax"] = 3
			end
		end	
	end

	if arg1 then
		StatusColorized = Crayon:Colorize(RGBPercToHex(RepFactionColors[RepIncreaseValue].r,
														RepFactionColors[RepIncreaseValue].g,
														RepFactionColors[RepIncreaseValue].b ),
														arg1)
														
		FactionColorized = Crayon:Colorize(RGBPercToHex(RepFactionColors[RepIncreaseValue].r,
														RepFactionColors[RepIncreaseValue].g,
														RepFactionColors[RepIncreaseValue].b ),
														arg2)
														
		if RepFactionsHolder[arg2] >= 4 then
			self:Pour(Crayon:Green("Congratulations!"), nil, nil, nil, nil, nil, nil, true, "NOTIFY", nil)
		else
			self:Pour(Crayon:Red("So Sad!"), nil, nil, nil, nil, nil, nil, true, "NOTIFY", nil)
		end
		self:Pour("You are now " .. StatusColorized .. " with " .. FactionColorized .. ".", nil, nil, nil, nil, nil, nil, true, "NOTIFY", RepFactionColors[RepExchange[arg1]].icon)
	end
end -- function ExpandTheReputation

-- Gets the Members in the AceComm Channel to communicate the known Random Dailies
function Dailies:GetChannelMembers()
	channelCount = GetNumDisplayChannels();
	
	for i=1, channelCount do
		channel, header, collapsed, channelNumber, count, active, category = GetChannelDisplayInfo(i);
		
		if (count) and (table.getn(DailiesRoster) ~= count) then
			GrabbedChannelList = false
		else
			GrabbedChannelList = true
		end
							
		if (count) then
			if (channel == "AceComm") then	
				if (GrabbedChannelList == false) then
					DailiesRoster = {}
					SetSelectedDisplayChannel(i)
					
					for y=1, count do
						local name, owner, moderator, muted, voiceEnabled = GetChannelRosterInfo(GetSelectedDisplayChannel(), y)
						DailiesRoster[y] = name
					end
				end
			end
		end
	end
end -- function GetChannelMembers

-- Sends the known Random Dailies
function Dailies:SendTheDailies()
	if DailyHInstance then
		Dailies:SendTheQuest("HInstance", DailyHInstance)
	end
	
	if DailyHInstanceWotLK then
		Dailies:SendTheQuest("HInstanceWotLK", DailiesCommHolder["HInstanceWotLK"])
	end
	
	if DailyDInstance then
		Dailies:SendTheQuest("DInstance", DailyDInstance)
	end

	if DailyDInstanceWotLK then
		Dailies:SendTheQuest("DInstanceWotLK", DailiesCommHolder["DInstanceWotLK"])
	end
	
	if DailyCooking then
		Dailies:SendTheQuest("Cooking", DailyCooking)
	end

	if DailyCookingWotLK then
		Dailies:SendTheQuest("CookingWotLK", DailiesCommHolder["CookingWotLK"])
	end
	
	if DailyFishing then
		Dailies:SendTheQuest("Fishing", DailyFishing)
	end

	if DailyBGPvP then
		Dailies:SendTheQuest("BGPvP", DailyBGPvP)
	end
	
	if DailyBGPvPWotLK then
		Dailies:SendTheQuest("BGPvPWotLK", DailiesCommHolder["BGPvPWotLK"])
	end	
	
	Dailies:SendTheUser("MEMBER", UnitName("player"), tonumber(Dailies.version))
end -- function SendTheDailies

-- Send the USER to see if they are using DQT
function Dailies:SendTheUser(CommUser, SenderName, DQTVersion)
	for i = 1, table.getn(DailiesRoster) do
		if AceLibrary("AceComm-2.0"):IsUserInChannel(DailiesRoster[i], "GLOBAL") then
			Dailies:SendCommMessage("WHISPER", DailiesRoster[i], CommUser, SenderName, DQTVersion)
		end
	end
end -- function SendTheUser

-- Send the Random Dailies to Unique Users
function Dailies:SendTheQuest(Instance, questName)
	for i = 1, table.getn(self.UniqueNames) do
		if (self.UniqueNames[i] and self.UniqueNames[i] ~= UnitName("player")) then
			if AceLibrary("AceComm-2.0"):IsUserInChannel(self.UniqueNames[i], "GLOBAL") then
				Dailies:Debug ("Sending " .. DailyQuestsList[questName].link .. " daily to: " .. Crayon:Gold(self.UniqueNames[i]))
				Dailies:SendCommMessage("WHISPER", self.UniqueNames[i], Instance, questName, tonumber(Dailies.version))
			else
				Dailies:Debug(Crayon:Red("Removing user: ") .. Crayon:Gold(self.UniqueNames[i]))
				table.remove(self.UniqueNames, i);
			end
		end
	end
end -- function SendTheQuest

-- Grab unique users for DQT
function Dailies:GetUniqueNames(SenderName)
	local doInsert = true;
	for i=1,table.getn(self.UniqueNames) do
		if self.UniqueNames[i] == SenderName then
			doInsert = false;
		end
	end	
	
	if doInsert then
		table.insert(self.UniqueNames, SenderName);
	end
end -- function GetUniqueUsers

function Dailies:GetSavedDailiesTime()
	local DailiesWeekDay, DailiesMonth, DailiesDay, DailiesYear = CalendarGetDate();
	local DailiesMonthName = DailiesCalendar_MONTH_NAMES[tonumber(DailiesMonth)]
	local DailiesDayName = DailiesCalendar_WEEKDAY_NAMES[tonumber(DailiesWeekDay)]

	local DailiesNumberDate = DailiesMonth .. ":" .. DailiesDay
	local DailiesDATE = DailiesDayName .. ", " .. DailiesMonthName .. " " .. DailiesDay .. ", " .. DailiesYear

	local QuestHour, QuestMinute = GetGameTime(); local TwentyFourTime = QuestHour .. ":" .. QuestMinute; local SavedDailiesTime = DailiesNumberDate .. ":" .. TwentyFourTime
	if not Dailies.db.server.SavedDailies.SavedTime then
		Dailies.db.server.SavedDailies.SavedTime = SavedDailiesTime
	end
end -- function GetSavedDailiesTime

-- Receives the dailies and collects the information
function Dailies.OnCommReceive:HInstance (prefix, sender, distribution, questName)
	for i=1,table.getn(self.UniqueNames) do
		if (self.UniqueNames[i] == sender) then
			Dailies:Debug (DailyQuestsList[questName].link .. " from: " .. Crayon:Gold(sender))
			Dailies.db.server.SavedDailies.HInstance = questName; Dailies:GetSavedDailiesTime()
			DailiesCommHolder["HInstance"] = questName; DailyHInstance = questName; DailiesKnown = true
		end
	end
end -- function Dailies.OnCommReceive:HInstance

function Dailies.OnCommReceive:DInstance (prefix, sender, distribution, questName)
	for i=1,table.getn(self.UniqueNames) do
		if (self.UniqueNames[i] == sender) then
			Dailies:Debug (DailyQuestsList[questName].link .. " from: " .. Crayon:Gold(sender))
			Dailies.db.server.SavedDailies.DInstance = questName; Dailies:GetSavedDailiesTime()
			DailiesCommHolder["DInstance"] = questName; DailyDInstance = questName; DailiesKnown = true
		end
	end
end -- function Dailies.OnCommReceive:DInstance

function Dailies.OnCommReceive:Cooking (prefix, sender, distribution, questName)
	for i=1,table.getn(self.UniqueNames) do
		if (self.UniqueNames[i] == sender) then
			Dailies:Debug (DailyQuestsList[questName].link .. " from: " .. Crayon:Gold(sender))
			Dailies.db.server.SavedDailies.Cooking = questName; Dailies:GetSavedDailiesTime()
			DailiesCommHolder["Cooking"] = questName; DailyCooking = questName; DailiesKnown = true
		end
	end
end -- function Dailies.OnCommReceive:Cooking

function Dailies.OnCommReceive:CookingWotLK (prefix, sender, distribution, questName)
	for i=1,table.getn(self.UniqueNames) do
		if (self.UniqueNames[i] == sender) then
			Dailies:Debug (DailyQuestsList[questName].link .. " from: " .. Crayon:Gold(sender))
			Dailies.db.server.SavedDailies.CookingWotLK = questName; Dailies:GetSavedDailiesTime()
			DailiesCommHolder["CookingWotLK"] = questName; DailyCookingWotLK = questName; DailiesKnownWotLK = true
		end
	end
end -- function Dailies.OnCommReceive:CookingWotLK

function Dailies.OnCommReceive:Fishing (prefix, sender, distribution, questName)
	for i=1,table.getn(self.UniqueNames) do
		if (self.UniqueNames[i] == sender) then
			Dailies:Debug (DailyQuestsList[questName].link .. " from: " .. Crayon:Gold(sender))
			Dailies.db.server.SavedDailies.Fishing = questName; Dailies:GetSavedDailiesTime()
			DailiesCommHolder["Fishing"] = questName; DailyFishing = questName; DailiesKnown = true
		end
	end
end -- function Dailies.OnCommReceive:Fishing

function Dailies.OnCommReceive:BGPvP (prefix, sender, distribution, questName)
	for i=1,table.getn(self.UniqueNames) do
		if (self.UniqueNames[i] == sender) then
			Dailies:Debug (DailyQuestsList[questName].link .. " from: " .. Crayon:Gold(sender))
			Dailies.db.server.SavedDailies.BGPvP = questName; Dailies:GetSavedDailiesTime()
			DailiesCommHolder["BGPvP"] = questName; DailyBGPvP = questName; DailiesKnown = true
		end
	end
end -- function Dailies.OnCommReceive:BGPvP

function Dailies.OnCommReceive:MEMBER (prefix, sender, distribution, UserName, DQTVersion)
	local doInsert = true;
	for i=1,table.getn(self.UniqueNames) do		
		if (self.UniqueNames[i] == UserName) then
			doInsert = false
		else
			if DQTVersion then
				if (tonumber(DQTVersion) < tonumber(Dailies.version)) then
					doInsert = false;
				end
			else
				doInsert = false;
			end		
		end		
	end	
	
	if doInsert then
		table.insert(self.UniqueNames, UserName);
		Dailies:Debug(Crayon:Red("Adding User: ") .. Crayon:Gold(UserName) .. " [V. " .. Crayon:Green(DQTVersion) .. "]")
	end
end -- function Dailies.OnCommReceive:MEMBER

-- Check if aquired new tradeskills and update gathering quests you can do
function Dailies:CHAT_MSG_SKILL(msg)
	local TradeskillGained = Deformat(msg, ERR_SKILL_GAINED_S)
	if TradeskillGained ==  GetSpellInfo(8613) or TradeskillGained == GetSpellInfo(2575) or TradeskillGained == GetSpellInfo(9134) then
		SSOCounter = 0;	TradeskillCounter = 0
		hasHerbalism = nil; hasMining = nil; hasSkinning = nil;
		Dailies:PLAYER_ALIVE()
	end
end -- function CHAT_MSG_SKILL

-- Cycle through Tradeskills
function Dailies:PLAYER_ALIVE()
	DQMax = GetMaxDailyQuests();
	DQComplete = GetDailyQuestsCompleted();
	numSkills = GetNumSkillLines();

	if TradeskillCounter == 0 then
		for i=1, numSkills do
			local skillName = GetSkillLineInfo(i)
			
			if skillName == GetSpellInfo(9134) then -- Herbalism
				DailyQuestsList[L["Gaining the Advantage"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_MISC_DUST_03", link = "|cffffff00|Hquest:11875:70|h[Gaining the Advantage]|h|r", coords = "62.811:35.576", qgiver = "Emissary Mordin", qzone = "Shattrath City", qszone = "Terrace of Light", group = "Shattered Sun Offensive", area = BZ["Outland"], SID = "Shattered Sun Offensive:4", ["rep"] = {BF["Shattered Sun Offensive"]..":250"}, ["rewards"] = {"18253:1"}, gold = 163900}
				DailyQuestsList[L["Netherdust Pollen"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Enchant_DustSoul", link = "|cffffff00|Hquest:11017:70|h[Netherdust Pollen]|h|r", coords ="66:86", qgiver = "Taskmaster Varkule Dragonbreath", qzone = "Shadowmoon Valley", qszone = "Dragonmaw Base Camp", group = "Netherwing", area = BZ["Shadowmoon Valley"], SID = "Netherwing:4", prof = GetSpellInfo(9134), ["rep"] = {BF["Netherwing"]..":250"}, gold = 119900} --Herbalism
				DailiesGroups["Netherwing"]["ValueMax"] = DailiesGroups["Netherwing"]["ValueMax"] + 1
				hasHerbalism = GetSpellInfo(9134)
				
			elseif skillName == GetSpellInfo(2575) then -- Mining
				DailyQuestsList[L["Gaining the Advantage"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_MISC_DUST_03", link = "|cffffff00|Hquest:11875:70|h[Gaining the Advantage]|h|r", coords = "62.811:35.576", qgiver = "Emissary Mordin", qzone = "Shattrath City", qszone = "Terrace of Light", group = "Shattered Sun Offensive", area = BZ["Outland"], SID = "Shattered Sun Offensive:4", ["rep"] = {BF["Shattered Sun Offensive"]..":250"}, ["rewards"] = {"18253:1"}, gold = 163900}			
				DailyQuestsList[L["Nethercite Ore"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Ore_Ethernium_01", link = "|cffffff00|Hquest:11018:70|h[Nethercite Ore]|h|r", coords ="66:86", qgiver = "Taskmaster Varkule Dragonbreath", qzone = "Shadowmoon Valley", qszone = "Dragonmaw Base Camp", group = "Netherwing", area = BZ["Shadowmoon Valley"], SID = "Netherwing:4", prof = GetSpellInfo(2575), ["rep"] = {BF["Netherwing"]..":250"}, gold = 119900} --Mining
				DailiesGroups["Netherwing"]["ValueMax"] = DailiesGroups["Netherwing"]["ValueMax"] + 1
				hasMining = GetSpellInfo(2575)
				
			elseif skillName == GetSpellInfo(8613) then -- Skinning
				DailyQuestsList[L["Gaining the Advantage"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_MISC_DUST_03", link = "|cffffff00|Hquest:11875:70|h[Gaining the Advantage]|h|r", coords = "62.811:35.576", qgiver = "Emissary Mordin", qzone = "Shattrath City", qszone = "Terrace of Light", group = "Shattered Sun Offensive", area = BZ["Outland"], SID = "Shattered Sun Offensive:4", ["rep"] = {BF["Shattered Sun Offensive"]..":250"}, ["rewards"] = {"18253:1"}, gold = 163900}
				DailyQuestsList[L["Nethermine Flayer Hide"]] = {XPac = "TBC", icon = "Interface\\Icons\\INV_Misc_Rune_12", link = "|cffffff00|Hquest:11016:70|h[Gaining the Advantage]|h|r", coords ="66:86", qgiver = "Taskmaster Varkule Dragonbreath", qzone = "Shadowmoon Valley", qszone = "Dragonmaw Base Camp", group = "Netherwing", area = BZ["Shadowmoon Valley"], SID = "Netherwing:4", prof = GetSpellInfo(8613), ["rep"] = {BF["Netherwing"]..":250"}, gold = 119900}  --Skinning
				DailiesGroups["Netherwing"]["ValueMax"] = DailiesGroups["Netherwing"]["ValueMax"] + 1
				hasSkinning = GetSpellInfo(8613)
				
			elseif skillName == GetSpellInfo(25229) then -- Jewelcrafting
				DailiesGroups["Jewelcrafting"].Profession = true
				
			elseif skillName == GetSpellInfo(2550) then -- Cooking
				DailiesGroups["Cooking"].Profession = true; DailiesGroups["CookingWotLK"].Profession = true

			elseif skillName == GetSpellInfo(7620) then -- Fishing
				DailiesGroups["Fishing"].Profession = true				
			end
			
			if SSOCounter == 0 then
				if (hasHerbalism or hasMining or hasSkinning) then
					DailiesGroups["Shattered Sun Offensive"]["ValueMax"] = DailiesGroups["Shattered Sun Offensive"]["ValueMax"] + 1
					SSOCounter = 1
				end
			end
			TradeskillCounter = 1
		end	
	end
end -- function PLAYER_ALIVE

-- FuBar Icon updates
function Dailies:OnTextUpdate()
	Dailies:SetIcon("Interface\\Addons\\DailiesQuestTracker\\Icons\\Numbers\\" .. (GetMaxDailyQuests() - GetDailyQuestsCompleted()) .. ".blp")
	
	DQColor = Crayon:GetThresholdHexColor((GetMaxDailyQuests() - GetDailyQuestsCompleted()), GetMaxDailyQuests())
	Dailies:SetText(" " .. Crayon:Colorize(DQColor, (GetMaxDailyQuests() - GetDailyQuestsCompleted())) .. "/" .. Crayon:Green(GetMaxDailyQuests()));
end -- function OnTextUpdate

-- Get Blizzards Dailies reset and break it apart
function Dailies:SecondsToTime(seconds, noSeconds, notAbbreviated)
	local time = "";
	local count = 0;
	local tempTime;
	seconds = floor(seconds);
	if ( seconds >= 86400  ) then
		tempTime = floor(seconds / 86400);
		if ( notAbbreviated ) then
			time = tempTime.." "..GetText("DAYS", nil, tempTime);
		else
			time = tempTime.." "..GetText("DAYS_ABBR", nil, tempTime);
		end
		seconds = mod(seconds, 86400);
		count = count + 1;
	end
	if ( seconds >= 3600  ) then
		if ( time ~= "" ) then
			time = time.." ";
		end
		tempTime = floor(seconds / 3600);
		if ( notAbbreviated ) then
			time = time..tempTime..":"
		else
			time = time..tempTime..":"
		end
		seconds = mod(seconds, 3600);
		count = count + 1;
	end
	if ( count < 2 and seconds >= 60  ) then
		if ( time ~= "" ) then
			time = time.." ";
		end
		tempTime = floor(seconds / 60);
		if ( notAbbreviated ) then
			time = time..tempTime..":"
		else
			time = time..tempTime..":"
		end
		seconds = mod(seconds, 60);
		count = count + 1;
	end
	if ( count < 2 and seconds > 0 and not noSeconds ) then
		if ( time ~= "" ) then
			time = time.." ";
		end
		seconds = format("%d", seconds);
		if ( notAbbreviated ) then
			time = time..seconds..":"
		else
			time = time..seconds..":"
		end
	end
	return time;
end -- function SecondsToTime


function Dailies:OnDataUpdate()
	local _, DailiesMonth, DailiesDay, _ = CalendarGetDate()
	local QuestHour, QuestMinute = GetGameTime()
	local CheckMonth = tonumber(DailiesMonth); CheckDay = tonumber(DailiesDay)
	
	if self.db.char.DailyQuestsCompleted then
		for UnitQuest, UnitQuestTime in pairs (self.db.char.DailyQuestsCompleted) do
			local UnitMonth, UnitDay, UnitHour, UnitMinute = (":"):split(UnitQuestTime)
			local UnitMonth = tonumber(UnitMonth); local UnitDay = tonumber(UnitDay); local UnitHour = tonumber(UnitHour); local UnitMinute = tonumber(UnitMinute)
			
			if ((CheckMonth == 1) and (CheckDay == 1) and (UnitHour >= DailiesReset) and (QuestMinute >= 0)) or
			((CheckMonth > UnitMonth) and (UnitHour <= DailiesReset) and (QuestHour >= DailiesReset) and (QuestMinute >= 0)) or
			((CheckDay > UnitDay) and (UnitHour >= DailiesReset) and (QuestHour >= DailiesReset) and (QuestMinute >= 0)) or
			((CheckDay == UnitDay) and (UnitHour <= DailiesReset) and (QuestHour >= DailiesReset) and (QuestMinute >= 0)) then
				self.db.char.DailyQuestsCompleted[UnitQuest] = nil
				self.db.char.GroupValues[DailyQuestsList[UnitQuest].group] = self.db.char.GroupValues[DailyQuestsList[UnitQuest].group] - 1
				Dailies:Debug ("Cleared Completed Daily " .. UnitQuest)
			end
		end
	end
	
	if Dailies.db.server.chars then
		for UnitName in pairs (Dailies.db.server.chars) do
			for UnitQuest in pairs (Dailies.db.server.chars[UnitName].UnitCompleted) do
				local UnitMonth, UnitDay, UnitHour, UnitMinute = (":"):split(Dailies.db.server.chars[UnitName].UnitCompleted[UnitQuest])
				local UnitMonth = tonumber(UnitMonth); local UnitDay = tonumber(UnitDay); local UnitHour = tonumber(UnitHour); local UnitMinute = tonumber(UnitMinute)
				--Dailies:Debug (UnitName .. "'s Unit Quest: " .. UnitQuest .. " | " .. UnitMonth .. "/" .. UnitDay .. " " .. UnitHour .. ":" .. UnitMinute)

				if ((CheckMonth == 1) and (CheckDay == 1) and (QuestHour >= DailiesReset) and (QuestMinute >= 0)) or
				((CheckMonth > UnitMonth) and (UnitHour >= DailiesReset) and (QuestMinute >= 0)) or
				((CheckDay > UnitDay) and (UnitHour >= DailiesReset) and (QuestHour >= DailiesReset) and (QuestMinute >= 0)) or
				((CheckDay == UnitDay) and (UnitHour <= DailiesReset) and (QuestHour >= DailiesReset) and (QuestMinute >= 0)) then
					Dailies.db.server.chars[UnitName].UnitCompleted[UnitQuest] = nil
					Dailies:Debug ("Cleared " .. UnitName .. "'s " .. UnitQuest)
				end
			end
		end
	end
	
	if Dailies.db.server.SavedDailies.SavedTime then
		local UnitMonth, UnitDay, UnitHour, UnitMinute = (":"):split(Dailies.db.server.SavedDailies.SavedTime)
		local UnitMonth = tonumber(UnitMonth); local UnitDay = tonumber(UnitDay); local UnitHour = tonumber(UnitHour); local UnitMinute = tonumber(UnitMinute)

		if ((CheckMonth == 1) and (CheckDay == 1) and (QuestHour >= DailiesReset) and (QuestMinute >= 0)) or
		((CheckMonth > UnitMonth) and (UnitHour >= DailiesReset) and (QuestMinute >= 0)) or
		((CheckDay > UnitDay) and (UnitHour >= DailiesReset) and (QuestHour >= DailiesReset) and (QuestMinute >= 0)) or
		((CheckDay == UnitDay) and (UnitHour <= DailiesReset) and (QuestHour >= DailiesReset) and (QuestMinute >= 0)) then
			Dailies.db.server.SavedDailies = {}
			Dailies:Debug ("Clearing Saved Dailies.")
		end	
	end
	
	if Dailies.db.server.SavedDailies.HInstance then
		DailyHInstance = Dailies.db.server.SavedDailies.HInstance; DailiesKnown = true
	end
	
	if Dailies.db.server.SavedDailies.DInstance then
		DailyDInstance = Dailies.db.server.SavedDailies.DInstance; DailiesKnown = true
	end
	
	if Dailies.db.server.SavedDailies.Cooking then
		DailyCooking = Dailies.db.server.SavedDailies.Cooking; DailiesKnown = true
	end
	
	if Dailies.db.server.SavedDailies.CookingWotLK then
		DailyCookingWotLK = Dailies.db.server.SavedDailies.CookingWotLK; DailiesKnownWotLK = true
	end	
	
	if Dailies.db.server.SavedDailies.Fishing then
		DailyFishing = Dailies.db.server.SavedDailies.Fishing; DailiesKnown = true
	end
	
	if Dailies.db.server.SavedDailies.BGPvP then
		DailyBGPvP = Dailies.db.server.SavedDailies.BGPvP; DailiesKnown = true
	end
end

function Dailies:OnTooltipUpdate()
	DailiesWeekDay, DailiesMonth, DailiesDay, DailiesYear = CalendarGetDate();
	DailiesMonthName = DailiesCalendar_MONTH_NAMES[tonumber(DailiesMonth)]
	DailiesDayName = DailiesCalendar_WEEKDAY_NAMES[tonumber(DailiesWeekDay)]

	DailiesNumberDate = DailiesMonth .. ":" .. DailiesDay
	DailiesDATE = DailiesDayName .. ", " .. DailiesMonthName .. " " .. DailiesDay .. ", " .. DailiesYear
	
	Tablet:SetTitle(L[" Dailies for "] .. DailiesDATE .. " ")
	englishFaction, _ = UnitFactionGroup("Player")
	
	TURN_IN_ICON = format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\ActiveQuestIcon.tga", Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0)
	ACCEPTED_ICON = format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\AvailableQuestIcon.tga", Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0)
	MAX_ICON = format(TEXTURE_LINK_FORMAT, "Interface\\Buttons\\UI-GroupLoot-Pass-Up", Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0)
	DQT_USERS_ICON = format(TEXTURE_LINK_FORMAT, "Interface\\Icons\\INV_Misc_GroupNeedMore", Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0)
	COMM_ON_ICON = format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\CommOn.tga", Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0)
	COMM_OFF_ICON = format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\CommOff.tga", Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0)
	
	if DailiesReset == 4 then
		DailiesResetSeconds = GetQuestResetTime() + 3600
	else
		DailiesResetSeconds = GetQuestResetTime()
	end
	
	local DailiesResetTimeHour, DailiesResetTimeMin = (":"):split(Dailies:SecondsToTime(DailiesResetSeconds, true, 1))
	local DailiesResetTimeHour = tonumber(DailiesResetTimeHour)
	local DailiesResetTimeMin = tonumber(DailiesResetTimeMin)
	
	local GameHour, GameMinute = GetGameTime()
	local LocalHour = tonumber(date("%H"))
	
	if HolidayHour and HolidayCalendarType == "HOLIDAY" then
		if (GameHour >= HolidayHour) and HolidaySequenceType == "END" then
			DailiesGroups["Holiday"].Profession = false
			Dailies:Debug("Turning off Holiday Information")
		end
	end
	
	if DailiesResetTimeHour then
		if DailiesResetTimeHour > 1 then
			HoursFormat = L["Hours"]
		else
			HoursFormat = L["Hour"]
		end
	end
	
	if DailiesResetTimeMin then
		if DailiesResetTimeMin > 1 then
			MinutesFormat = L["Minutes"]
		else
			MinutesFormat = L["Minute"]
		end
	end
	
	if DailiesResetSeconds >= 3600 then
		DailiesIconError = false
		
		-- GameTime/LocalTime offset
		if self.db.char.TooltipTimer == true then
			UserTimerChoice = " [" .. Crayon:Green(L["LOCAL"]) .. "]: "
			if (GameHour >= LocalHour) then
				DailiesResetTimeHour = (DailiesResetTimeHour) - (GameHour - LocalHour)
			end
		else
			UserTimerChoice = " [" .. Crayon:Green(L["REALM"]) .. "]: "
		end

		if DailiesResetTimeMin then
			DailiesText = (UserTimerChoice .. L["Dailies are scheduled to reset in "] .. Crayon:Green(DailiesResetTimeHour .. " " .. HoursFormat) .. L[" and "] .. Crayon:Green(DailiesResetTimeMin .. " " .. MinutesFormat) .. L["."])
		else
			DailiesText = (UserTimerChoice .. L["Dailies are scheduled to reset in "] .. Crayon:Green(DailiesResetTimeHour .. " " .. HoursFormat) .. L["."])
		end
	else
		if DailiesResetTimeMin then
			DailiesText = (UserTimerChoice .. L["Dailies are scheduled to reset in "] .. Crayon:Green(DailiesResetTimeMin .. " " .. MinutesFormat) .. L["."])
		else
			DailiesText = Crayon:Red(L["[ERROR]: "]) .. "|cffAA0000" .. L["Reset time from realm is not valid."] .. "|r"
			DailiesIconError = true
		end
	end

	local DQTooltipColor = Crayon:GetThresholdHexColor((GetMaxDailyQuests() - GetDailyQuestsCompleted()), GetMaxDailyQuests())
	if ( DQComplete < DQMax ) then
		-- You can complete XX daily quests today
		AmountText = L["You can complete "] .. Crayon:Colorize(DQTooltipColor, (GetMaxDailyQuests() - GetDailyQuestsCompleted())) ..L[" daily quests today."]
	else
		-- You cannot complete any more daily quests today
		AmountText = L["You cannot complete any more daily quests today."]
	end
	
	if table.getn(self.UniqueNames) <= 0 then
		UniqueUsers = Crayon:Red("0")
	else
		UniqueUsers = Crayon:Green(table.getn(self.UniqueNames))
	end	
	
	cat = Tablet:AddCategory(
		'columns', 3,
		'showWithoutChildren', true,
		'hideBlankLine', true
	 )
			
	if self.db.char.DailyQuestsCompleted then
		for ExpansionName, _  in pairsByKeys (ExpansionGroups) do
				--Dailies:Debug(tostring(self.db.char[ExpansionName]))
				--Dailies:Debug(tostring(ExpansionGroups[ExpansionName].expansionLevel) .. " " .. XPacLevel)
				
				if self.db.char[ExpansionName] and (ExpansionGroups[ExpansionName].expansionLevel <= XPacLevel) then
					cat = Tablet:AddCategory(
						'text', format(TEXTURE_LINK_FORMAT, [[Interface\Addons\DailiesQuestTracker\Icons\]] .. ExpansionName .. [[.tga]], 13, 13, 0) .. ExpansionGroups[ExpansionName].title,
						'textR', 1, 'textG', 0.8, 'textB', 0,
						'size', 13,
						'text2', "",
						'text2R', 1, 'text2G', 0.8, 'text2B', 0,
						'columns', 2,
						'showWithoutChildren', true,
						'hideBlankLine', true,
						'func', function() self.db.profile.hideDetails[ExpansionGroups[ExpansionName]["group"]] = not self.db.profile.hideDetails[ExpansionGroups[ExpansionName]["group"]] end,
						'hasCheck', true,
						'checked', true,
						'checkIcon', self.db.profile.hideDetails[ExpansionGroups[ExpansionName]["group"]] and "Interface\\Buttons\\UI-PlusButton-Up" or "Interface\\Buttons\\UI-MinusButton-Up"
					)
				
					for GroupName, GroupText in pairsByKeys (DailiesGroups) do
						if self.db.char.Filters[GroupName] == false and DailiesGroups[GroupName].Profession == true and self.db.profile.hideDetails[ExpansionGroups[ExpansionName]["group"]]  == true and (RepFactionsHolder[GroupName] >= 4) and DailiesGroups[GroupName].XPac == ExpansionName then
							
							--ChatFrame3:AddMessage(GroupName .. " " .. tostring(Dailies.db.char.GroupValues[GroupName]))
							--ChatFrame3:AddMessage(GroupName .. " " .. tostring(DailiesGroups[GroupName].ValueMax))
							
							MinColor = Crayon:GetThresholdHexColor(self.db.char.GroupValues[GroupName], DailiesGroups[GroupName].ValueMax)

							if (RepValuesHolder[GroupName]) and (RepValuesHolder[GroupName].title ~= "Unknown") then
								FactionColorize = Crayon:Colorize(RGBPercToHex(RepFactionColors[RepValuesHolder[GroupName].StandingID].r,
																							   RepFactionColors[RepValuesHolder[GroupName].StandingID].g,
																							   RepFactionColors[RepValuesHolder[GroupName].StandingID].b),
																							   RepFactionColors[RepValuesHolder[GroupName].StandingID].title)
								FactionThreshold = Crayon:GetThresholdHexColor(RepValuesHolder[GroupName].ReputationIs, RepValuesHolder[GroupName].ReputationMax)
								
								FactionValue = Crayon:Colorize(FactionThreshold, Dailies:Commas(RepValuesHolder[GroupName].ReputationIs)) .. " of " .. Crayon:Green(Dailies:Commas(RepValuesHolder[GroupName].ReputationMax))
								
								if self.db.char.FactionDisplay == "Icon" then
									FactionFormattedSuffix = ": [ " .. format(TEXTURE_LINK_FORMAT, RepFactionColors[RepValuesHolder[GroupName].StandingID].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. "]"
								elseif self.db.char.FactionDisplay == "Value" then
									FactionFormattedSuffix = ": [" .. FactionColorize .. ": " .. FactionValue .. "]"
								elseif self.db.char.FactionDisplay == "Title" then
									FactionFormattedSuffix = ": [" .. FactionColorize .. "]"
								end
							else
								FactionFormattedSuffix = ""
							end
							
							cat = Tablet:AddCategory(
								'indentation', 14,
								'text', DailiesGroups[GroupName]["title"] .. FactionFormattedSuffix,
								'textR', 1, 'textG', 0.8, 'textB', 0,
								'wrap', false,
								'text2', "[" .. Crayon:Colorize(MinColor, self.db.char.GroupValues[GroupName]) .. "/" .. "|cff2ba1ff" .. DailiesGroups[GroupName].Accepted .. "|r" .. "/" .. Crayon:Green(DailiesGroups[GroupName]["ValueMax"]) .. "]",
								'text2R', 1, 'text2G', 0.8, 'text2B', 0,
								'wrap2', false,
								'columns', 2,
								'showWithoutChildren', true,
								'hideBlankLine', true,
								'func', function() self.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] = not self.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] end,
								'hasCheck', true,
								'checked', true,
								'checkIcon', self.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] and "Interface\\Buttons\\UI-PlusButton-Up" or "Interface\\Buttons\\UI-MinusButton-Up"
							)
							
							for index, value in pairsByKeys (DailyQuestsList) do
								if Dailies.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] == false then		
									if DailyQuestsList[index]["group"] == GroupName then
									
										TotalUnits = 0; UnitsCompleted = 0; DailyUnitsTotal = ""
										for PlayerName, NilHolder in pairsByKeys (Dailies.db.server.chars) do
											if Dailies.db.server.chars[PlayerName].UnitCanDo[index] then
												TotalUnits = TotalUnits + 1	
												UnitsHolder[index] = TotalUnits
											end
											
											if Dailies.db.server.chars[PlayerName].UnitCompleted[index] then
												UnitsCompleted = UnitsCompleted + 1
											end
											
											UnitMinColor = Crayon:GetThresholdHexColor(UnitsCompleted, UnitsHolder[index])
											if (UnitsHolder[index]) and (UnitsHolder[index] >= 2) then
												DailyUnitsTotal = Crayon:Gold("[") .. Crayon:Colorize(UnitMinColor, UnitsCompleted) .. Crayon:Gold("/") .. Crayon:Green(UnitsHolder[index]) .. Crayon:Gold("] ")
											else
												DailyUnitsTotal = ""
											end
										end							
									
										if self.db.char.DailyQuestsObjectives[index] == "Turn-In" then RColor= 1; GColor= 1; BColor = 0; RIcon= 1; GIcon= 1; BIcon = 1; CompletedText = TURN_IN_ICON
											elseif self.db.char.DailyQuestsCompleted[index] then RColor= 1; GColor= 1; BColor = 1; RIcon= 1; GIcon= 1; BIcon = 1;
											
												if self.db.char.TimeFormat == "Standard" then
													local QuestMonth, QuestDay, QuestHour, QuestMinute = (":"):split(self.db.char.DailyQuestsCompleted[index])
													local QuestPM = floor(QuestHour / 12) == 1; QuestHour = mod(QuestHour, 12); if QuestHour == 0 then QuestHour = 12 end;
													local QuestTime = string.format("%d" .. Crayon:Gold(":") .. "%02d" .. Crayon:Gold(" %s"), QuestHour, QuestMinute, QuestPM and "PM" or "AM");	
													CompletedText = QuestTime
												else
													local QuestMonth, QuestDay, QuestHour, QuestMinute = (":"):split(self.db.char.DailyQuestsCompleted[index])
													CompletedText = string.format("%d" .. Crayon:Gold(":") .. "%02d", QuestHour, QuestMinute)
												end
											
											elseif self.db.char.DailyQuestsAccepted[index] then RColor= .17; GColor= .63; BColor = 1; RIcon= 1; GIcon= 1; BIcon = 1; CompletedText = ACCEPTED_ICON
											elseif self.db.char.GroupValues[DailiesGroups[GroupName]["group"]] ==  DailiesGroups[GroupName]["ValueMax"] then RColor= 1; GColor= 0; BColor = 0; RIcon= 1; GIcon= 0; BIcon = 0; CompletedText = MAX_ICON
											else RColor= 0.5; GColor= 0.5; BColor = 0.5; RIcon= 0.5; GIcon= 0.5; BIcon = 0.5;
											
											if DailyQuestsList[index]["dates"] then
												CompletedText = DailyQuestsList[index]["dates"].starting .." - ".. DailyQuestsList[index]["dates"].ending
											elseif DailyQuestsList[index]["loc"] then
												CompletedText = DailyQuestsList[index]["loc"]
											elseif (GetDailyQuestsCompleted() == GetMaxDailyQuests()) then
												RColor= 1; GColor= 0; BColor = 0; RIcon= 1; GIcon= 0; BIcon = 0;
												CompletedText = MAX_ICON
											else
												CompletedText = " "
											end
										end
																	
										local ReputationName, ReputationID = (":"):split(DailyQuestsList[index]["SID"]); ReputationID = tonumber(ReputationID)
										if (index and RepFactionsHolder[ReputationName]) and (ReputationID <= RepFactionsHolder[ReputationName])  then
											cat:AddLine(									
												'text', " " .. DailyUnitsTotal .. index,
												'textR', RColor,
												'textG', GColor,
												'textB', BColor,
												'indentation', 28,
												'text2', CompletedText,
												'text2R', RColor,
												'text2G', GColor,
												'text2B', BColor,
												'hasCheck', true,
												'checked', true,
												'checkIcon', DailyQuestsList[index]["icon"],
												'checkColorR', RIcon,
												'checkColorG', BIcon,
												'checkColorB', GIcon,
												'onEnterFunc', function()
													Dailies:OnHoverTooltip(index, GetMouseFocus():GetName(),  "ENTER")
												end,
												'onLeaveFunc', function()
													Dailies:OnHoverTooltip(nil, nil, "LEAVE")
												end,
												'func', 'OnQuestClick',
												'arg1', self,
												'arg2', index
											)
										end
									end
								end
							end
							if Dailies.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] == false then cat:AddLine () end
						end
					end
				end
			end
	
	if (XPacLevel == 0) then
		cat = Tablet:AddCategory(
			'columns', 1,
			'showWithoutChildren', true,
			'hideBlankLine', true
		 )

		cat:AddLine(									
			'text', " ",
			'justify', "CENTER"
			)
			
		cat:AddLine(									
			'text', "You need " .. Crayon:Green("The Burning Crusade") .. " or " .. Crayon:Blue("Wrath of the Lich King") .. " expansions",
			'justify', "CENTER"
			)
			
		cat:AddLine(									
			'text', "in order the use Dailies Quest Tracker",
			'justify', "CENTER"
			)

		cat:AddLine(									
			'text', " ",
			'justify', "CENTER"
			)			
	end
	
	if (self.db.char.DailyCommSet == true) then
		if (self.db.char.HideDailyList == false) and (XPacLevel > 0) then
			if (DailiesKnown == nil) and (DailiesKnownWotLK == nil) then
				cat = Tablet:AddCategory(
					'columns', 1,
					'showWithoutChildren', true,
					'hideBlankLine', true
				 )
				 
				cat:AddLine(
					'text', " "
				)
			end
			
			-- Random Dailies Report List for The Burning Crusade
			if (DailiesKnown ~= nil) and (self.db.char.TBC == true) then
				-- Spacer	
				cat = Tablet:AddCategory(
					'columns', 1,
					'showWithoutChildren', true,
					'hideBlankLine', true
				 )

				cat:AddLine(
					'text', " "
				)
				
				cat:AddLine(				
					'text', Crayon:Green("The Burning Crusade") .. " [" .. Crayon:Green("Random Dailies").. "]" .. L[" are:"],
					'justify', "CENTER"
				)
				
				cat = Tablet:AddCategory(
					'columns', 3,
					'showWithoutChildren', true,
					'hideBlankLine', true
				 )

				if DailyBGPvP then
					if Dailies.db.char.DailyQuestsCompleted[DailyBGPvP] then
						DailiesKnownBGPvP = DailiesKnownCompleted
					else
						DailiesKnownBGPvP = DailiesKnownNotCompleted
					end
					cat:AddLine(					
					'text', Crayon:White(DailyBGPvP),
					'text2', "[" .. DailiesKnownBGPvP .."]",
					'text3', "[" .. Crayon:Green(L["Daily Battleground"]) .."]",
					'hasCheck', true,
					'checked', true,
					'checkIcon', DailyQuestsList[DailyBGPvP]["icon"],
					'func', 'QuestClickLink',
					'arg1', self,
					'arg2', DailyBGPvP				
					)
				end
				
				if DailyCooking then
					if Dailies.db.char.DailyQuestsCompleted[DailyCooking] then
						DailiesKnownCooking = DailiesKnownCompleted
					else
						DailiesKnownCooking = DailiesKnownNotCompleted
					end
					cat:AddLine(					
					'text', Crayon:White(DailyCooking),
					'text2', "[" .. DailiesKnownCooking .."]",
					'text3', "[" .. Crayon:Green(L["Daily Cooking"]) .."]",
					'hasCheck', true,
					'checked', true,
					'checkIcon', DailyQuestsList[DailyCooking]["icon"],
					'func', 'QuestClickLink',
					'arg1', self,
					'arg2', DailyCooking
					)
				end

				if DailyDInstance then
					if Dailies.db.char.DailyQuestsCompleted[DailyDInstance] then
						DailiesKnownDInstance = DailiesKnownCompleted
					else
						DailiesKnownDInstance = DailiesKnownNotCompleted
					end
					cat:AddLine(					
					'text', Crayon:White(DailyDInstance),
					'text2', "[" .. DailiesKnownDInstance .."]",
					'text3', "[" .. Crayon:Green(L["Daily Dungeon"]) .."]",
					'hasCheck', true,
					'checked', true,
					'checkIcon', DailyQuestsList[DailyDInstance]["icon"],
					'func', 'QuestClickLink',
					'arg1', self,
					'arg2', DailyDInstance				
					)
				end
				
				if DailyFishing then
					if Dailies.db.char.DailyQuestsCompleted[DailyFishing] then
						DailiesKnownFishing = DailiesKnownCompleted
					else
						DailiesKnownFishing = DailiesKnownNotCompleted
					end
					cat:AddLine(					
					'text', Crayon:White(DailyFishing),
					'text2', "[" .. DailiesKnownFishing .."]",
					'text3', "[" .. Crayon:Green(L["Daily Fishing"]) .."]",
					'hasCheck', true,
					'checked', true,
					'checkIcon', DailyQuestsList[DailyFishing]["icon"],
					'func', 'QuestClickLink',
					'arg1', self,
					'arg2', DailyFishing				
					)
				end
				
				if DailyHInstance then
					if Dailies.db.char.DailyQuestsCompleted[DailyHInstance] then
						DailiesKnownHInstance = DailiesKnownCompleted
					else
						DailiesKnownHInstance = DailiesKnownNotCompleted
					end
					cat:AddLine(					
					'text', Crayon:White(DailyHInstance),
					'text2', "[" .. DailiesKnownHInstance .."]",
					'text3', "[" .. Crayon:Green(L["Daily Heroic"]) .."]",
					'hasCheck', true,
					'checked', true,
					'checkIcon', DailyQuestsList[DailyHInstance]["icon"],
					'func', 'QuestClickLink',
					'arg1', self,
					'arg2', DailyHInstance
					)
				end
								
				if not self.db.char.WotLK then
					cat:AddLine(
						'text', " "
					)
				end
				
			end
			
			-- Random Dailies Report List for Wrath of the Lich King
			if (DailiesKnownWotLK ~= nil) and (self.db.char.WotLK) then
				-- Spacer	
				cat = Tablet:AddCategory(
					'columns', 1,
					'showWithoutChildren', true,
					'hideBlankLine', true
				 )
				 
				cat:AddLine(
					'text', " "
				)
				
				cat:AddLine(				
					'text', Crayon:Blue("Wrath of the Lich King") .. " [" .. Crayon:Green("Random Dailies").. "]" .. L[" are:"],
					'justify', "CENTER"
				)

				cat = Tablet:AddCategory(
					'columns', 3,
					'showWithoutChildren', true,
					'hideBlankLine', true
				 )
				 
				if DailyCookingWotLK then
					if Dailies.db.char.DailyQuestsCompleted[DailyCookingWotLK] then
						DailiesKnownCooking = DailiesKnownCompleted
					else
						DailiesKnownCooking = DailiesKnownNotCompleted
					end
					cat:AddLine(					
					'text', Crayon:White(DailyCookingWotLK),
					'text2', "[" .. DailiesKnownCooking .."]",
					'text3', "[" .. Crayon:Green(L["Daily Cooking"]) .."]",
					'hasCheck', true,
					'checked', true,
					'checkIcon', DailyQuestsList[DailyCookingWotLK]["icon"],
					'func', 'QuestClickLink',
					'arg1', self,
					'arg2', DailyCookingWotLK
					)
				end
				
				cat:AddLine(
					'text', " "
				)				
			end
			
		end
	end
	
		if self.db.char.DailyCommSet == true then
			cat = Tablet:AddCategory(
				'columns', 3,
				'showWithoutChildren', true,
				'hideBlankLine', true
			 )		
			
			cat:AddLine(
			'text', COMM_ON_ICON .. ": " .. Crayon:Green(L["ON"]),
			'text2', AmountText,
			'text3', DQT_USERS_ICON .. ": " .. UniqueUsers .. "  ",
			'justify2', 'CENTER'
			)
		else
			cat = Tablet:AddCategory(
				'columns', 3,
				'showWithoutChildren', true,
				'hideBlankLine', true
			 )

			 cat:AddLine(
			'text', COMM_OFF_ICON .. ": " .. Crayon:Red(L["OFF"]),
			'text2', AmountText,
			'text3', DQT_USERS_ICON .. ": " .. Crayon:Red(L["OFF"]) .. "  ",
			'justify2', 'CENTER'
			)		
		end
	end

	cat = Tablet:AddCategory(
		'columns', 1,
		'showWithoutChildren', true,
		'hideBlankLine', true
	 )
			 
	
	-- Displays when the dailies will reset
	cat:AddLine(		
		'text', DailiesText,
		'justify', "CENTER"
	)
end -- function OnTooltipUpdate

function Dailies:OnHoverTooltip(QuestTitleValue, Focus, Status)
	if Status == "ENTER" then
		if self.db.char.TooltipPlacement == "Left" then
			QuestRewardsTooltip:SetOwner(getglobal(Focus), "ANCHOR_BOTTOMRIGHT", 12, 14)
		else
			QuestRewardsTooltip:SetOwner(getglobal(Focus), "ANCHOR_BOTTOMLEFT", -12, 14)
		end
		
		if self.db.char.DailyQuestsCompleted[QuestTitleValue] then
			-- Return with no colorization
		elseif Dailies.db.char.DailyQuestsObjectives[QuestTitleValue] == "Turn-In" then
			QuestRewardsTooltip:SetBackdropColor(.50, .50, 0)
			QuestRewardsTooltip:SetBackdropBorderColor(1, 1, 0)
		elseif self.db.char.DailyQuestsAccepted[QuestTitleValue] then
			QuestRewardsTooltip:SetBackdropColor(0, .13, .50)
			QuestRewardsTooltip:SetBackdropBorderColor(.17, .63, 1)
		elseif self.db.char.GroupValues[DailyQuestsList[QuestTitleValue].group] ==  DailiesGroups[DailyQuestsList[QuestTitleValue].group]["ValueMax"] then
			QuestRewardsTooltip:SetBackdropColor(.50, 0, 0)
			QuestRewardsTooltip:SetBackdropBorderColor(1, 0, 0)
		else
			QuestRewardsTooltip:SetBackdropColor(.09, .09, .19)
			QuestRewardsTooltip:SetBackdropBorderColor(1, 1, 1)
		end
		QuestRewardsTooltip:AddLine(QuestTitleValue)
		
		-- If ITEMS are awarded, add them to the tooltip
		if DailyQuestsList[QuestTitleValue]["rewards"] then
			for Reward, RewardType in pairs(DailyQuestsList[QuestTitleValue]["rewards"]) do
				-- Grab itemlink and quantity from quest and break it apart
				local RewardType, RewardAmount = (":"):split(RewardType); RewardAmount = tonumber(RewardAmount)
				if RewardAmount < 2 then
					RewardAmount = ""
				else
					RewardAmount = (" x" .. Crayon:White(RewardAmount))
				end
				
				local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(RewardType); 
				if itemLink == nil then
					GameTooltip:SetHyperlink("item:".. RewardType .. ":0:0:0:0:0:0:0")
				end
				
				if itemLink then
					QuestRewardsTooltip:AddLine(itemLink .. RewardAmount)
					QuestRewardsTooltip:AddTexture(itemTexture)
				end
			end
		end

		-- If HONOR awarded, add it to the tooltip
		if DailyQuestsList[QuestTitleValue]["honor"] then
			QuestRewardsTooltip:AddLine(Crayon:White("~") .. Crayon:Green(DailyQuestsList[QuestTitleValue]["honor"]) .. Crayon:White(L[" Honor Points"]))
			QuestRewardsTooltip:AddTexture(BGHonor)
		end

		-- If REPUTATION awarded, add it to the tooltip
		if DailyQuestsList[QuestTitleValue]["rep"] then	
			for RepReward, RepRewardType in pairs(DailyQuestsList[QuestTitleValue]["rep"]) do
			-- Grab faction name and reputation from quest and break it apart
				local RepRewardType, RepAmount = (":"):split(RepRewardType)
				-- Check to see if the PLAYER is HUMAN and multiply Reputation by 10%
				if PlayerRace == "Human" then RepAmount = ceil(RepAmount + (RepAmount * .10)) else RepAmount = RepAmount end
				-- If Repuation Standing is nil then set it to UNKNOWN
				if RepFactionsHolder[RepRewardType] == nil then RepFactionsHolder[RepRewardType] = 0 end

				QuestRewardsTooltip:AddLine(Crayon:Green(RepAmount) .. Crayon:White(L[" reputation with "]) .. Crayon:Colorize(RGBPercToHex(RepFactionColors[RepFactionsHolder[RepRewardType]].r,
																										RepFactionColors[RepFactionsHolder[RepRewardType]].g,
																										RepFactionColors[RepFactionsHolder[RepRewardType]].b ),
																										RepRewardType))
				QuestRewardsTooltip:AddTexture("Interface\\Addons\\DailiesQuestTracker\\Icons\\Rep\\" .. RepRewardType)

				if RepFactionsHolder[RepRewardType] == 0 then RepFactionsHolder[RepRewardType] = nil end
			end
		end
		
		-- If GOLD awarded, add it to the tooltip
		if DailyQuestsList[QuestTitleValue]["gold"] then
			MoneyFormat = Dailies:FormatTheGold(DailyQuestsList[QuestTitleValue]["gold"], 0)
			QuestRewardsTooltip:AddLine(MoneyFormat)
			QuestRewardsTooltip:AddTexture("Interface\\Icons\\INV_Misc_Coin_02")
		end
		
		-- Get Quest Giver information, add it to the tooltip
		if (Dailies.db.char.QuestGiver == true) then
			if DailyQuestsList[QuestTitleValue]["coords"] then
				local LeftCoord, RightCoord = (":"):split(DailyQuestsList[QuestTitleValue]["coords"]); local LeftCoord = tonumber(LeftCoord); local RightCoord = tonumber (RightCoord)
				QuestRewardsTooltip:AddLine(" ") -- Spacer
				
				QuestRewardsTooltip:AddLine(QuestTitleValue .. " Quest Giver:")
				QuestRewardsTooltip:AddLine(Crayon:White(DailyQuestsList[QuestTitleValue]["qgiver"]))
				QuestRewardsTooltip:AddTexture("Interface\\Addons\\DailiesQuestTracker\\Icons\\AvailableQuestIcon.tga")
				
				if (DailyQuestsList[QuestTitleValue]["qszone"] == "Nil") then
					QuestRewardsTooltip:AddLine(Crayon:White(DailyQuestsList[QuestTitleValue]["qzone"]))
					QuestRewardsTooltip:AddTexture("Interface\\TaxiFrame\\TaxiMap530")
				else
					QuestRewardsTooltip:AddLine(((Crayon:White(DailyQuestsList[QuestTitleValue]["qszone"]) .. Crayon:Silver(", ")) or "") .. Crayon:White(DailyQuestsList[QuestTitleValue]["qzone"]))
					QuestRewardsTooltip:AddTexture("Interface\\TaxiFrame\\TaxiMap530")				
				end
								
				if (LeftCoord == 0) and (RightCoord == 0) then
					-- Add no Coordinates
				else
					QuestRewardsTooltip:AddLine(Crayon:White("Coordinates: ") .. Crayon:Green(ceil(LeftCoord)) .. Crayon:Silver(", ") .. Crayon:Green(ceil(RightCoord)))
					QuestRewardsTooltip:AddTexture("Interface\\Icons\\INV_Misc_PocketWatch_02")
				end
			end
		end		
		QuestRewardsTooltip:Show()
		
		if (self.db.char.MutilpleCharacters == true) then
			QuestUnitsTooltip:SetBackdropColor(.09, .09, .19)
			QuestUnitsTooltip:SetBackdropBorderColor(1, 1, 1)
			
			if UnitsHolder[QuestTitleValue] then
				if self.db.char.TooltipPlacement == "Left" then
					QuestUnitsTooltip:SetOwner(QuestRewardsTooltip, "ANCHOR_NONE")
					QuestUnitsTooltip:SetPoint("TOPLEFT", QuestRewardsTooltip, "BOTTOMLEFT")
				else
					QuestUnitsTooltip:SetOwner(QuestRewardsTooltip, "ANCHOR_NONE")
					QuestUnitsTooltip:SetPoint("TOPRIGHT", QuestRewardsTooltip, "BOTTOMRIGHT")
				end

				QuestUnitsTooltip:SetText("Character(s) on [" .. Crayon:Green(GetRealmName()) .. "]:")	
				
				if (self.db.server.chars) and (self.db.server.UnitTrackerValid == 1) and (UnitsHolder[QuestTitleValue] >= 2) then
					for PlayerName, NilHolder in pairsByKeys(Dailies.db.server.chars) do
						if (PlayerName ~= UnitName("player")) then
							if (Dailies.db.server.chars[PlayerName].UnitCanDo[QuestTitleValue]) then
								
								if Dailies.db.server.chars[PlayerName].UnitCompleted[QuestTitleValue] then
									UnitQuestStatus = DailiesKnownCompleted
								else
									UnitQuestStatus = DailiesKnownNotCompleted
								end
								
								local PClass, PFaction = (":"):split(Dailies.db.server.chars[PlayerName].PlayerClass)
								
								if PFaction == FACTION_ALLIANCE then
									FactionIcon = "Interface\\Addons\\DailiesQuestTracker\\Icons\\Alliance.tga"
									FactionIconName = Crayon:Green(FACTION_ALLIANCE)
								else
									FactionIcon = "Interface\\Addons\\DailiesQuestTracker\\Icons\\Horde.tga"
									FactionIconName = Crayon:Green(FACTION_HORDE)
								end
								
								local FACTION_ICON = format(TEXTURE_LINK_FORMAT, FactionIcon, 12, 12, -1)							
								local PlayerNameHEX = RGBPercToHex(RAID_CLASS_COLORS[PClass].r, RAID_CLASS_COLORS[PClass].g, RAID_CLASS_COLORS[PClass].b)
								
								QuestUnitsTooltip:AddDoubleLine(Crayon:Colorize(PlayerNameHEX, PlayerName) .. " [" .. FACTION_ICON .. FactionIconName .. "]", Crayon:Gold(" [") .. UnitQuestStatus .. Crayon:Gold("]"))
								QuestUnitsTooltip:AddTexture(classIcons[PClass])
								QuestUnitsTooltip:Show()
							end
						end
					end
				else
					QuestUnitsTooltip:Hide()
				end
			end
		else
			QuestUnitsTooltip:Hide()
		end
	elseif Status == "LEAVE" then
		QuestUnitsTooltip:Hide()
		QuestRewardsTooltip:Hide()
	end
end

function Dailies:OnQuestClick(QuestTitleValue)
	if IsShiftKeyDown() then
		local LeftCoord, RightCoord = (":"):split(DailyQuestsList[QuestTitleValue]["coords"])
		Dailies:CoordClickLink(LeftCoord, RightCoord, QuestTitleValue)
	elseif IsControlKeyDown() then
		Dailies:QuestClickLinkShowLog(QuestTitleValue)
	else
		Dailies:QuestClickLink(QuestTitleValue)
	end
end -- function OnQuestClick

-- Format the gold with icons for money placement
function Dailies:FormatTheGold (GoldAmount, Offset)
local gold = tonumber(abs(floor(GoldAmount / 10000)))
local silver = tonumber(abs(floor(mod(GoldAmount / 100, 100))))
local copper = tonumber(abs(floor(mod(GoldAmount, 100))))

GOLD_ICON = format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\GoldIcon.tga", 12, 12, Offset)
SILVER_ICON = format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\SilverIcon.tga", 12, 12, Offset)
COPPER_ICON = format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\CopperIcon.tga", 12, 12, Offset)

if (copper == 0) then CopperFormat = "" else CopperFormat = Crayon:Copper(copper) .. COPPER_ICON .. " " end
if (silver == 0) then SilverFormat = "" else SilverFormat = Crayon:Silver(silver) .. SILVER_ICON .. " " end
if (gold == 0) then GoldFormat = "" else GoldFormat = Crayon:Gold(gold) .. GOLD_ICON .. " " end

MoneyFormat = GoldFormat .. SilverFormat .. CopperFormat
return MoneyFormat
end -- function FormatTheGold

-- Click on FuBar icon will toggle the Quest Log Window
function Dailies:OnClick(button)
	if ( QuestLogFrame:IsShown() ) then
		HideUIPanel(QuestLogFrame);
	else
		ShowUIPanel(QuestLogFrame);
	end
end -- function OnClick

-- Makes the tooltip an actual item
function Dailies:SetTooltip(arg1, arg2)
	GameTooltip_SetDefaultAnchor(GameTooltip, this)
	GameTooltip:SetHyperlink("item:"..arg1..":0:0:0:0:0:0:0")
end -- function SetTooltip

--Control-Left-Click to show quest in QuestLog
function Dailies:QuestClickLinkShowLog(QuestTitle)
	local numEntries = GetNumQuestLogEntries()
	
	if numEntries > 0 then
		for QuestID = 1, numEntries do
			local QuestName, _, _, _, QuestHeader, QuestCollapsed, QuestCompleted, QuestIsDaily = GetQuestLogTitle(QuestID)
			if (QuestName == QuestTitle) and (Dailies.db.char.DailyQuestsAccepted[QuestName]) then
				if (QuestLogFrame:IsShown()) then
					SelectQuestLogEntry(QuestID)
					QuestLog_SetSelection(QuestID)
				else
					SelectQuestLogEntry(QuestID)
					QuestLog_SetSelection(QuestID)
					ShowUIPanel(QuestLogFrame)
				end
			end
		end
		--QuestLog_Update();
	end
end

-- Shift-Left-Click to paste the quest link into the chatbox
function Dailies:QuestClickLink(link)
	if not ChatFrameEditBox:IsVisible() then
		ChatFrameEditBox:Show()
	end
	
	if DailyQuestsList[link]["loc"] then
		if DailyQuestsList[link]["link"] then
			ChatFrameEditBox:Insert(DailyQuestsList[link]["link"] .. L[" located in "].. DailyQuestsList[link]["loc"])
		else
			ChatFrameEditBox:Insert(L["No Quest Link known!"])
		end
	else
		if DailyQuestsList[link]["link"] then
			ChatFrameEditBox:Insert(DailyQuestsList[link]["link"])
		else
			ChatFrameEditBox:Insert(L["No Quest Link known!"])
		end
	end
	
	-- Get Quest HyperLink and send it to break apart
	local numEntries = GetNumQuestLogEntries();
	local questLogTitleText;
	for i=1, numEntries, 1 do
		questLogTitleText = GetQuestLogTitle(i);
		if ( questLogTitleText == link ) then
			Dailies:GetQuestDetails(GetQuestLink(i));
		end
	end
end -- function QuestClickLink

-- Click to bring up Waypoint depending on addon installed
function Dailies:CoordClickLink(LeftCoord, RightCoord, QuestName)
	local CoordsZoneName = GetRealZoneText()
	if (LeftCoord and RightCoord) and (self.db.char.TOMTOM_WAYPOINTS == true) then
		if (CoordsZoneName == DailyQuestsList[QuestName]["qzone"]) then
			-- TomTom:AddWaypoint(x, y, desc, persistent, minimap, world)
			TomTom:AddWaypoint(LeftCoord, RightCoord, Crayon:Green(DailyQuestsList[QuestName]["qgiver"]) .. ", " .. DailyQuestsList[QuestName].link, nil, nil, nil)
		end
	end

	if (LeftCoord and RightCoord) and (self.db.char.CART_WAYPOINTS == true) then
		if (CoordsZoneName == DailyQuestsList[QuestName]["qzone"]) then
			-- Cartographer_Waypoints:AddLHWaypoint(c, z, x, y, desc)
			Cartographer_Waypoints:AddLHWaypoint(nil, nil, LeftCoord, RightCoord, Crayon:Green(DailyQuestsList[QuestName]["qgiver"]) .. ", " .. DailyQuestsList[QuestName].link)
		end
	end
	
	if (LeftCoord and RightCoord) and (self.db.char.CART_WAYPOINTS_THREE == true) then
		if (CoordsZoneName == DailyQuestsList[QuestName]["qzone"]) then
			Cartographer3_Waypoints.SetWaypoint(CoordsZoneName, LeftCoord, RightCoord, Crayon:Green(DailyQuestsList[QuestName]["qgiver"]) .. ", " .. DailyQuestsList[QuestName].link, "POI")
		end
	end	
end -- function CoordClickLink

-- Click to paste itemlink into the chatbox
function Dailies:itemClickLink(link, amount)
	if not ChatFrameEditBox:IsVisible() then
		ChatFrameEditBox:Show()
	end
	
	if link then
		ChatFrameEditBox:Insert(link .. amount)
	end
end -- function itemClickLink

-- Gather Random Dailies when the quest-giver GOSSIP is shown
function Dailies:GOSSIP_SHOW()
	local englishFaction, _ = UnitFactionGroup("Player")
	local _, DPlayerClass = UnitClass("player")
	local QuestName, _, _ = GetGossipAvailableQuests()
	
	if QuestName and DailyQuestsList[QuestName] then
		Dailies:Debug(L["Gathered quest: "] .. QuestName)
		Dailies.db.server.chars[UnitName("player")].UnitCanDo[QuestName] = true; DailiesPlayerClass = (DPlayerClass .. ":" .. englishFaction); Dailies.db.server.chars[UnitName("player")].PlayerClass = DailiesPlayerClass
		-- The Burning Crusade Dailies
		if DailyQuestsList[QuestName]["group"] == "HInstance" then DailyHInstance = QuestName; Dailies.db.server.SavedDailies.HInstance = QuestName; DailiesKnown = true; DailiesCommHolder["HInstance"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "DInstance" then DailyDInstance = QuestName; Dailies.db.server.SavedDailies.DInstance = QuestName; DailiesKnown = true; DailiesCommHolder["DInstance"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "Cooking" then DailyCooking = QuestName; Dailies.db.server.SavedDailies.Cooking = QuestName; DailiesKnown = true; DailiesCommHolder["Cooking"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "Fishing" then DailyFishing = QuestName; Dailies.db.server.SavedDailies.Fishing = QuestName; DailiesKnown = true; DailiesCommHolder["Fishing"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "BGPvP" then DailyBGPvP = QuestName; Dailies.db.server.SavedDailies.BGPvP = QuestName; DailiesKnown = true; DailiesCommHolder["BGPvP"] = QuestName end

		-- Wrath of the Lich King Dailies
		if DailyQuestsList[QuestName]["group"] == "HInstanceWotLK" then DailyHInstanceWotLK = QuestName; Dailies.db.server.SavedDailies.HInstanceWotLK = QuestName; DailiesKnownWotLK = true; DailiesCommHolder["HInstanceWotLK"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "DInstanceWotLK" then DailyDInstanceWotLK = QuestName; Dailies.db.server.SavedDailies.DInstanceWotLK = QuestName; DailiesKnownWotLK = true; DailiesCommHolder["DInstanceWotLK"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "CookingWotLK" then DailyCookingWotLK = QuestName; Dailies.db.server.SavedDailies.CookingWotLK = QuestName; DailiesKnownWotLK = true; DailiesCommHolder["CookingWotLK"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "FishingWotLK" then DailyFishingWotLK = QuestName; Dailies.db.server.SavedDailies.FishingWotLK = QuestName; DailiesKnownWotLK = true; DailiesCommHolder["FishingWotLK"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "BGPvPWotLK" then DailyBGPvPWotLK = QuestName; Dailies.db.server.SavedDailies.BGPvPWotLK = QuestName; DailiesKnownWotLK = true; DailiesCommHolder["BGPvPWotLK"] = QuestName end		
		
		Dailies:GetSavedDailiesTime()	
	end
end -- function GOSSIP_SHOW

-- Gather Random Dailies when the quest-giver DETAILS are shown
function Dailies:QUEST_DETAIL()
	local englishFaction, _ = UnitFactionGroup("Player")
	local _, DPlayerClass = UnitClass("player")
	local QuestName = GetTitleText()
	
	if QuestName and DailyQuestsList[QuestName] then
		Dailies:Debug(L["Gathered quest: "] .. QuestName)
		Dailies.db.server.chars[UnitName("player")].UnitCanDo[QuestName] = true; DailiesPlayerClass = (DPlayerClass .. ":" .. englishFaction); Dailies.db.server.chars[UnitName("player")].PlayerClass = DailiesPlayerClass
		-- The Burning Crusade Dailies
		if DailyQuestsList[QuestName]["group"] == "HInstance" then DailyHInstance = QuestName; Dailies.db.server.SavedDailies.HInstance = QuestName; DailiesKnown = true; DailiesCommHolder["HInstance"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "DInstance" then DailyDInstance = QuestName; Dailies.db.server.SavedDailies.DInstance = QuestName; DailiesKnown = true; DailiesCommHolder["DInstance"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "Cooking" then DailyCooking = QuestName; Dailies.db.server.SavedDailies.Cooking = QuestName; DailiesKnown = true; DailiesCommHolder["Cooking"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "Fishing" then DailyFishing = QuestName; Dailies.db.server.SavedDailies.Fishing = QuestName; DailiesKnown = true; DailiesCommHolder["Fishing"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "BGPvP" then DailyBGPvP = QuestName; Dailies.db.server.SavedDailies.BGPvP = QuestName; DailiesKnown = true; DailiesCommHolder["BGPvP"] = QuestName end

		-- Wrath of the Lich King Dailies
		if DailyQuestsList[QuestName]["group"] == "HInstanceWotLK" then DailyHInstanceWotLK = QuestName; Dailies.db.server.SavedDailies.HInstanceWotLK = QuestName; DailiesKnownWotLK = true; DailiesCommHolder["HInstanceWotLK"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "DInstanceWotLK" then DailyDInstanceWotLK = QuestName; Dailies.db.server.SavedDailies.DInstanceWotLK = QuestName; DailiesKnownWotLK = true; DailiesCommHolder["DInstanceWotLK"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "CookingWotLK" then DailyCookingWotLK = QuestName; Dailies.db.server.SavedDailies.CookingWotLK = QuestName; DailiesKnownWotLK = true; DailiesCommHolder["CookingWotLK"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "FishingWotLK" then DailyFishingWotLK = QuestName; Dailies.db.server.SavedDailies.FishingWotLK = QuestName; DailiesKnownWotLK = true; DailiesCommHolder["FishingWotLK"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "BGPvPWotLK" then DailyBGPvPWotLK = QuestName; Dailies.db.server.SavedDailies.BGPvPWotLK = QuestName; DailiesKnownWotLK = true; DailiesCommHolder["BGPvPWotLK"] = QuestName end		

		Dailies:GetSavedDailiesTime()
	end
end -- function QUEST_DETAIL

-- Add commas for every thousandths place
function Dailies:Commas(num)
	assert (type (num) == "number" or type (num) == "string")

	local result = ""
	local sign, before, after = string.match (tostring (num), "^([%+%-]?)(%d*)(%.?.*)$")

	-- pull out batches of 3 digits from the end, put a comma before them
	while string.len (before) > 3 do
		result = "," .. string.sub (before, -3, -1) .. result
		before = string.sub (before, 1, -4)  -- remove last 3 digits
	end -- while

	return sign .. before .. result .. after
end -- function Commas

-- Break apart QuestLink to get data
function Dailies:GetQuestDetails(questString)
	if not questString then return nil end

	if DailiesDebugEnabled == true then	
		printable = gsub(questString, "|", "||")
		DEFAULT_CHAT_FRAME:AddMessage(printable)
	end
end -- function GetQuestDetails

-- Dailies Quest Tracker options menu
function Dailies:OnMenuRequest(level, value, inTooltip)		
    if not inTooltip then
		if level == 1 then
			-- Dailies Config Title
			Dewdrop:AddLine(
				'text', L["Dailies Config"],
				'isTitle', true,
				'icon', "Interface\\Addons\\DailiesQuestTracker\\Icon.tga",
				'iconWidth', 12,
				'iconHeight', 12
			)

			-- Random Dailies Communication Toggle
			Dewdrop:AddLine(
				'text', L["Random Dailies Communication"],
				'checked', self.db.char.DailyCommSet,
				'func', function ()
					SetDailyCommSet  = not SetDailyCommSet
					if (SetDailyCommSet  == true) then
						StaticPopup_Show ("RESET_COMMS", DialogOnComms)
					elseif (SetDailyCommSet  == false) then
						StaticPopup_Show ("RESET_COMMS", DialogOffComms)
					end
				end,
				'isRadio', true,
				'tooltipTitle', L["Random Dailies Communication"],
				'tooltipText', L["This toggles turning communications on/off for Dailies Quest Tracker."]
			)
			
			-- Hides/Shows the Random Dailies List
			Dewdrop:AddLine(
				'text', L["Hide Random Dailies List"],
				'checked', self.db.char.HideDailyList,
				'disabled', not self.db.char.DailyCommSet,
				'func', function ()
					self.db.char.HideDailyList  = not self.db.char.HideDailyList
				end,
				'isRadio', true,
				'tooltipTitle', L["Hide Random Dailies List"],
				'tooltipText', L["Hides the random dailies list in Dailies Quest Tracker."]
			)			
			
			-- Location unfold
			Dewdrop:AddLine(
				'text', L["Auto Unfold Groups"],
				'checked', self.db.char.LocationShow,
				'func', function ()
					self.db.char.LocationShow  = not self.db.char.LocationShow
				end,
				'isRadio', true,
				'tooltipTitle', L["Auto Unfold Groups"],
				'tooltipText', L["Automatically unfolds groups when you are in the zone or instance of those quests."]
			)

			-- Location unfold
			Dewdrop:AddLine(
				'text', "Multiple Characters",
				'checked', self.db.char.MutilpleCharacters,
				'func', function ()
					self.db.char.MutilpleCharacters  = not self.db.char.MutilpleCharacters
				end,
				'isRadio', true,
				'tooltipTitle', "Multiple Characters",
				'tooltipText', "Displays tooltips for dailies known by all characters on [" .. Crayon:Green(GetRealmName()).. "]."
			)
			
			-- QuestGiver locations
			Dewdrop:AddLine(
				'text', "Quest Givers",			
			--	'text', format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\AvailableQuestIcon.tga", Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), MENU_OFFSET) .. "Quest Givers",
				'checked', self.db.char.QuestGiver,
				'func', function ()
					self.db.char.QuestGiver  = not self.db.char.QuestGiver
				end,
				'isRadio', true,
				'tooltipTitle', "Quest Givers",
				'tooltipText', "Shows the Quest Givers location in the Dailies Quest Tracker tooltips."
			)			
			
			-- Time Offset
			Dewdrop:AddLine(
				'text', L["Dailies Reset Time Offset"],
				'checked', self.db.char.TooltipTimer,
				'disabled', true,				
				'func', function ()
					self.db.char.TooltipTimer  = not self.db.char.TooltipTimer
				end,
				'isRadio', true,
				'tooltipTitle', L["Dailies Reset Time Offset"],
				'tooltipText', L["Toggles the visual timer in the tooltip to the REALM or your LOCAL time."]
			)
			
			-- Spacer
			Dewdrop:AddSeparator()
			
			-- AceComm Joining Channel Delay
			Dewdrop:AddLine(
			'text', "Communications Delay",
			'hasArrow', true,
			'hasSlider', true,
			'sliderMin', 10,
			'sliderMinText', '10 Secs.',
			'sliderMax', 60,
			'sliderMaxText', '60 Secs.',
			'sliderStep', 5,
			'sliderValue', self.db.profile.AceCommDelay,
			'sliderFunc', function(value)
		        self.db.profile.AceCommDelay = value
		    end,
			'icon', "Interface\\Addons\\DailiesQuestTracker\\Icons\\CommOn.tga",
			'tooltipTitle', "Communications Delay",
			'tooltipText', "Sets the delay when joining the AceComm Channel. Setting this value higher, will help with older computers having issues with AceComm joining Channel 1.\n\nThis setting will take effect the next time you log-in the game or reload your user interface."
			)	
			
			if XPacLevel == 0 then
				ShowIcon = "Interface\\Icons\\INV_Misc_QuestionMark"
			elseif XPacLevel == 2 then
				ShowIcon = "Interface\\Addons\\DailiesQuestTracker\\Icons\\WotLK.tga"
			else
				ShowIcon = "Interface\\Addons\\DailiesQuestTracker\\Icons\\TBC.tga"
			end
			
			-- Expansions Display
			Dewdrop:AddLine(
				'text', "Expansions",
				'icon', ShowIcon,
				'value', "ExpansionDisplay",
				'hasArrow', true,
				'tooltipTitle', ""
			)
			
			-- Faction Display
			Dewdrop:AddLine(
				'text', L["Faction Display"],
				'hasArrow', true,
				'icon', "Interface\\PvPRankBadges\\PvPRank15",
				'value', "FactionDisplay",
				'tooltipTitle', L["Faction Display"],
				'tooltipText', "Lets you change how the faction values are displayed in the Dailies Quest Tracker Tooltip."
			)	
			
			-- Filters
			Dewdrop:AddLine(
				'text', L["Filters"],
				'hasArrow', true,
				'icon', "Interface\\Buttons\\UI-GroupLoot-Pass-Up",
				'value', "Filters",
				'tooltipTitle', L["Filters"],
				'tooltipText', L["Lets you filter out groups you have no interest in watching."]
			)
					
			-- Tooltip Placement
			Dewdrop:AddLine(
				'text', L["Rewards Tooltip Placement"],
				'hasArrow', true,
				'value', "Rewards",
				'icon', "Interface\\Addons\\DailiesQuestTracker\\Icons\\Tooltip.tga",
				'tooltipTitle',  L["Rewards Tooltip Placement"],
				'tooltipText', L["Places the rewards tooltip on either the Left or Right of Dailies Quest Tracker."]
			)			
				
			-- Show Me The Money!
			Dewdrop:AddLine(
				'text', L["Show Me The Money!"],
				'hasArrow', true,
				'value', "MoneyRewards",
				'icon', "Interface\\Icons\\INV_Misc_Coin_02",
				'tooltipTitle',  L["Show Me The Money!"],
				'tooltipText', L["Makes a list of the highest monetary value of the dailies."]
			)			
			
			-- Scrolling Combat Text Title
			Dewdrop:AddLine(
			'text', L["Scrolling Combat Text"],
			'hasArrow', true,
			'value', "SCT",
			'icon', "Interface\\Addons\\DailiesQuestTracker\\Icons\\SCT.tga",
			'tooltipTitle', L["Scrolling Combat Text"],
			'tooltipText', L["Displays notifications if you have specific addon's installed."]
			)
			
			Dewdrop:AddLine(
			'text', "Time Format",
			'hasArrow', true,
			'value', "TimeFormat",
			'icon', "Interface\\Icons\\INV_Misc_PocketWatch_03",
			'tooltipTitle', "Time Format",
			'tooltipText', "Lets you display the time in Standard AM/PM or 24 hour format."
			)			
			
			-- Waypoints
			if (TOMTOM_LOADED == true) or (CARTOGRAPH_LOADED == true) or (CARTOGRAPH_THREE_LOADED == true) then
				Dewdrop:AddLine(
				'text', "Waypoints",
				'hasArrow', true,
				'value', "Waypoints",
				'icon', "Interface\\TaxiFrame\\TaxiMap530",
				'checkColorR', 1,
				'checkColorG', 0,
				'checkColorB', 0,				
				'tooltipTitle', "Waypoints",
				'tooltipText', "Lets you show waypoints for quest givers when " .. Crayon:Green("Shift-Left-Clicking") .. " quest titles, if you have specific mapping mods installed."
				)
			end			
					
			-- Spacer
			Dewdrop:AddSeparator()
			
			-- HINTS
			Dewdrop:AddLine(
				'text', Crayon:Green(L["[Hints]"]),
				'hasArrow', true,
				'icon', "Interface\\GossipFrame\\AvailableQuestIcon",
				'value', "Hints",
				'tooltipTitle', Crayon:Green(L["[Hints]"]),
				'tooltipText', L["Hints for specific things done in Dailies Quest Tracker."]
			)			

			-- Spacer
			Dewdrop:AddSeparator()

		elseif level == 3 then
			if value == "ScrollArea" then
				Dewdrop:FeedAceOptionsTable(self:GetSinkAce2OptionsDataTable().output, 1)
			end
			
		elseif level == 2 then								
			if value == "ExpansionDisplay" then
				ExpansionsFrame:Show()
				
				if not GetMouseFocus() then
					ExpansionsFrame:Hide()
				else
					if not Dailies:IsHooked(_G[GetMouseFocus():GetName()], "OnLeave") then
						Dailies:HookScript(_G[GetMouseFocus():GetName()], "OnLeave", function(this)
							Dailies.hooks[this].OnLeave(this)
							ExpansionsFrame:Hide()
						end)
					end
				end
								
				if TBCExpansion == true then
					TBCText = Crayon:Green("The Burning Crusade")
				else
					TBCText = "|cff008000" .. "The Burning Crusade" .. "|r"
				end
				
				Dewdrop:AddLine(
				'text', TBC_ICON .. TBCText,
				'checked', self.db.char.TBC,				
				'disabled', not TBCExpansion,
				'func', function ()
					self.db.char.TBC = not self.db.char.TBC
				end,
				'isRadio', true,
				'tooltipTitle', format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\TBC.tga", 32, 32, 0) .. " " .. TBCText,
				'tooltipText', "Shows dailies for The Burning Crusade in the Dailies Quest Tracker tooltip."				
				)
				
				if WotLKExpansion == true then
					WotLKText = Crayon:Blue("Wrath of the Lich King")
				else
					WotLKText = "|cff000080" .. "Wrath of the Lich King" .. "|r"
				end
					
				Dewdrop:AddLine(
				'text', WotLK_ICON .. WotLKText,
				'checked', self.db.char.WotLK,
				'disabled', not WotLKExpansion,
				'func', function ()
					self.db.char.WotLK = not self.db.char.WotLK
				end,
				'isRadio', true,
				'tooltipTitle', format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\WotLK.tga", 32, 32, 0) .. " " .. WotLKText,
				'tooltipText', "Shows dailies for The Wrath of the Lich King in the Dailies Quest Tracker tooltip."
				)
			end
			
			if value == "FactionDisplay" then
				Dewdrop:AddLine(
				'text', L["Icon"] .. ":" .. Crayon:Gold(" [" .. format(TEXTURE_LINK_FORMAT, "Interface\\PvPRankBadges\\PvPRank15", 12, 12, 0) .. "]"),
				'checked', self.db.char.FactionDisplay == "Icon",
				'func', function () 
					self.db.char.FactionDisplay = "Icon"
				end,
				'isRadio', true,
				'tooltipFunc', function()
					IconFrame:ClearAllPoints()
					IconFrame:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMRIGHT", -CONTAINER_OFFSET_X - 13, CONTAINER_OFFSET_Y);
					IconFrame:Show()

					if not GetMouseFocus() then
						IconFrame:Hide()
					else
						if not Dailies:IsHooked(_G[GetMouseFocus():GetName()], "OnLeave") then
							Dailies:HookScript(_G[GetMouseFocus():GetName()], "OnLeave", function(this)
								Dailies.hooks[this].OnLeave(this)
								IconFrame:Hide()
							end)
						end
					end
				end,
				'tooltipTitle', ""
				)
				
				Dewdrop:AddLine(
				'text', L["Title"] .. ":" .. Crayon:Gold(" [") .. Crayon:Colorize(RGBPercToHex(0.6, 0.2, 0.8), "Exalted") .. Crayon:Gold("]"),
				'checked', self.db.char.FactionDisplay == "Title",
				'func', function () 
					self.db.char.FactionDisplay = "Title"
				end,
				'isRadio', true,
				'tooltipTitle', "Title",
				'tooltipText', "Sets the group faction status with Titles."
				)

				Dewdrop:AddLine(
				'text', L["Value"] .. ":" .. Crayon:Gold(" [") .. Crayon:Colorize(RGBPercToHex(0.6, 0.2, 0.8), "Exalted") .. Crayon:Gold(": ").. Crayon:Red("0") .. Crayon:Gold(" of ") .. Crayon:Green("1,000") .. Crayon:Gold("]"),
				'checked', self.db.char.FactionDisplay == "Value",
				'func', function ()
					self.db.char.FactionDisplay = "Value"
				end,
				'isRadio', true,
				'tooltipTitle', "Value",
				'tooltipText', "Sets the group faction status with Values."
				)				
			end
			
			if value == "Rewards" then
				Dewdrop:AddLine(
				'text', L["Left"],
				'checked', self.db.char.TooltipPlacement == "Right",
				'func', function ()
					RightTooltip = true; LeftTooltip = nil; 
					self.db.char.TooltipPlacement = "Right"
				end,
				'isRadio', true,
				'tooltipTitle', "Left",
				'tooltipText', "Place the rewards tooltip on the Left of Dailies Quest Tracker."
				)
				
				Dewdrop:AddLine(
				'text', L["Right"],
				'checked', self.db.char.TooltipPlacement == "Left",
				'func', function ()
					RightTooltip = nil; LeftTooltip = true; 
					self.db.char.TooltipPlacement = "Left"
				end,
				'isRadio', true,
				'tooltipTitle', "Right",
				'tooltipText', "Place the rewards tooltip on the Right of Dailies Quest Tracker."				
				)
			end
			
			if value == "TimeFormat" then
				Dewdrop:AddLine(
				'text', "Standard",
				'checked', self.db.char.TimeFormat == "Standard",
				'func', function ()
					self.db.char.TimeFormat = "Standard"
				end,
				'isRadio', true,
				'tooltipTitle', "Standard",
				'tooltipText', "The time will display with the AM/PM format."
				)
				
				Dewdrop:AddLine(
				'text', "24 Hour",
				'checked', self.db.char.TimeFormat == "TwentyFourHour",
				'func', function ()
					self.db.char.TimeFormat = "TwentyFourHour"
				end,
				'isRadio', true,
				'tooltipTitle', "24 Hour",
				'tooltipText', "The time will display with the 24 hour format."		
				)
			end			

			if value == "Filters" then
				if self.db.char.TBC then
					Dewdrop:AddLine(
						'text', Crayon:Green("The Burning Crusade"),
						'isTitle', true,
						'icon', [[Interface\Addons\DailiesQuestTracker\Icons\TBC.tga]]
					)				
			
					Dewdrop:AddLine(
					'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["BGPvP"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. "Battleground",
					'checked', self.db.char.Filters["BGPvP"],
					'func', function ()
						self.db.char.Filters["BGPvP"] = not self.db.char.Filters["BGPvP"]
					end,
					'isRadio', true
					)
					
					if DailiesGroups["Cooking"].Profession == true then
						Dewdrop:AddLine(
						'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["Cooking"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. GetSpellInfo(2550),
						'checked', self.db.char.Filters["Cooking"],
						'func', function ()
							self.db.char.Filters["Cooking"] = not self.db.char.Filters["Cooking"]				
						end,
						'isRadio', true
						)
					end

					Dewdrop:AddLine(
					'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["DInstance"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. "Dungeon Instance",
					'checked', self.db.char.Filters["DInstance"],
					'func', function ()
						self.db.char.Filters["DInstance"] = not self.db.char.Filters["DInstance"]				
					end,
					'isRadio', true
					)
			
					if DailiesGroups["Fishing"].Profession == true then
						Dewdrop:AddLine(
						'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["Fishing"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. GetSpellInfo(7620),
						'checked', self.db.char.Filters["Fishing"],
						'func', function ()
							self.db.char.Filters["Fishing"] = not self.db.char.Filters["Fishing"]				
						end,
						'isRadio', true
						)
					end
					
					Dewdrop:AddLine(
					'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["HInstance"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. "Heroic Instance",
					'checked', self.db.char.Filters["HInstance"],
					'func', function ()
						self.db.char.Filters["HInstance"] = not self.db.char.Filters["HInstance"]				
					end,
					'isRadio', true
					)				
					
					if DailiesGroups["Holiday"].Profession == true then
						Dewdrop:AddLine(
						'text', HolidayIcon .. " " .. "Holiday",
						'checked', self.db.char.Filters["Holiday"],
						'func', function ()
							self.db.char.Filters["Holiday"] = not self.db.char.Filters["Holiday"]				
						end,
						'isRadio', true
						)
					end
					

					if (RepFactionsHolder["Netherwing"] >= 4) then
						Dewdrop:AddLine(
						'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["Netherwing"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. "Netherwing",
						'checked', self.db.char.Filters["Netherwing"],
						'func', function ()
							self.db.char.Filters["Netherwing"] = not self.db.char.Filters["Netherwing"]				
						end,
						'isRadio', true
						)
					end

					if (RepFactionsHolder["Ogri'la"] >= 4) then
						Dewdrop:AddLine(
						'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["Ogri'la"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. "Ogri'la",
						'checked', self.db.char.Filters["Ogri'la"],
						'func', function ()
							self.db.char.Filters["Ogri'la"] = not self.db.char.Filters["Ogri'la"]				
						end,
						'isRadio', true
						)
					end

					if (RepFactionsHolder["Sha'tari Skyguard"] >= 4) then
						Dewdrop:AddLine(
						'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["Sha'tari Skyguard"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. "Sha'tari Skyguard",
						'checked', self.db.char.Filters["Sha'tari Skyguard"],
						'func', function ()
							self.db.char.Filters["Sha'tari Skyguard"] = not self.db.char.Filters["Sha'tari Skyguard"]				
						end,
						'isRadio', true
						)
					end
					
					if (RepFactionsHolder["Shattered Sun Offensive"] >= 4) then
						Dewdrop:AddLine(
						'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["Shattered Sun Offensive"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. "Shattered Sun Offensive",
						'checked', self.db.char.Filters["Shattered Sun Offensive"],
						'func', function ()
							self.db.char.Filters["Shattered Sun Offensive"] = not self.db.char.Filters["Shattered Sun Offensive"]				
						end,
						'isRadio', true
						)
					end		

					Dewdrop:AddLine(
					'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["World PvP"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. "World PvP",
					'checked', self.db.char.Filters["World PvP"],
					'func', function ()
						self.db.char.Filters["World PvP"] = not self.db.char.Filters["World PvP"]				
					end,
					'isRadio', true
					)
				end
				
				if self.db.char.WotLK then
					if self.db.char.TBC then
						Dewdrop:AddSeparator()
					end
					
					Dewdrop:AddLine(
						'text', Crayon:Blue("Wrath of the Lich King"),
						'isTitle', true,
						'icon', [[Interface\Addons\DailiesQuestTracker\Icons\WotLK.tga]]
					)	
					
					if DailiesGroups["CookingWotLK"].Profession == true then
						Dewdrop:AddLine(
						'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["Cooking"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. GetSpellInfo(2550),
						'checked', self.db.char.Filters["CookingWotLK"],
						'func', function ()
							self.db.char.Filters["CookingWotLK"] = not self.db.char.Filters["CookingWotLK"]				
						end,
						'isRadio', true
						)
					end

					Dewdrop:AddLine(
					'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["DInstanceWotLK"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. "Dungeon Instance",
					'checked', self.db.char.Filters["DInstanceWotLK"],
					'func', function ()
						self.db.char.Filters["DInstanceWotLK"] = not self.db.char.Filters["DInstanceWotLK"]				
					end,
					'isRadio', true
					)					
					
					if (RepFactionsHolder["Frenzyheart Tribe"] > 4) then
						Dewdrop:AddLine(
						'text', "Frenzyheart Tribe",
						'checked', self.db.char.Filters["Frenzyheart Tribe"],
						'func', function ()
							self.db.char.Filters["Frenzyheart Tribe"] = not self.db.char.Filters["Frenzyheart Tribe"]				
						end,
						'isRadio', true
						)
					end							

					Dewdrop:AddLine(
					'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["HInstanceWotLK"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. "Heroic Instance",
					'checked', self.db.char.Filters["HInstanceWotLK"],
					'func', function ()
						self.db.char.Filters["HInstanceWotLK"] = not self.db.char.Filters["HInstanceWotLK"]				
					end,
					'isRadio', true
					)						
					
					if DailiesGroups["Jewelcrafting"].Profession == true then
						Dewdrop:AddLine(
						'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["Jewelcrafting"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. GetSpellInfo(25229),
						'checked', self.db.char.Filters["Jewelcrafting"],
						'func', function ()
							self.db.char.Filters["Jewelcrafting"] = not self.db.char.Filters["Jewelcrafting"]				
						end,
						'isRadio', true
						)
					end

					if RepFactionsHolder["Northrend"] then
						Dewdrop:AddLine(
						'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["Northrend"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. "Northrend",
						'checked', self.db.char.Filters["Northrend"],
						'func', function ()
							self.db.char.Filters["Northrend"] = not self.db.char.Filters["Northrend"]				
						end,
						'isRadio', true
						)
					end		
					
					if (RepFactionsHolder["The Kalu'ak"] >= 4) then
						Dewdrop:AddLine(
						'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["The Kalu'ak"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. "The Kalu'ak",
						'checked', self.db.char.Filters["The Kalu'ak"],
						'func', function ()
							self.db.char.Filters["The Kalu'ak"] = not self.db.char.Filters["The Kalu'ak"]				
						end,
						'isRadio', true
						)
					end	
					
					if (RepFactionsHolder["The Oracles"] >= 4) then
						Dewdrop:AddLine(
						'text', "The Oracles",
						'checked', self.db.char.Filters["The Oracles"],
						'func', function ()
							self.db.char.Filters["The Oracles"] = not self.db.char.Filters["The Oracles"]				
						end,
						'isRadio', true
						)
					end

					Dewdrop:AddLine(
					'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["WorldPvPWotLK"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. "World PvP",
					'checked', self.db.char.Filters["WorldPvPWotLK"],
					'func', function ()
						self.db.char.Filters["WorldPvPWotLK"] = not self.db.char.Filters["WorldPvPWotLK"]				
					end,
					'isRadio', true
					)
					
					if (RepFactionsHolder["The Wyrmrest Accord"] >= 4) then
						Dewdrop:AddLine(
						'text', format(TEXTURE_LINK_FORMAT, DailiesGroups["The Wyrmrest Accord"].icon, Tablet:GetNormalFontSize(), Tablet:GetNormalFontSize(), 0) .. " " .. "The Wyrmrest Accord",
						'checked', self.db.char.Filters["The Wyrmrest Accord"],
						'func', function ()
							self.db.char.Filters["The Wyrmrest Accord"] = not self.db.char.Filters["The Wyrmrest Accord"]				
						end,
						'isRadio', true
						)
					end					
				end
			end
			
			if value == "SCT" then
				Dewdrop:FeedAceOptionsTable(self:GetSinkAce2OptionsDataTable().output, 1)
			end
			
			if value == "Hints" then
				Dewdrop:AddLine(
				'icon', "Interface\\GossipFrame\\AvailableQuestIcon",
				'text', Crayon:Green(L["Left-Click"]) .. L[" on the FuBar icon to open the Quest Log Window."]
				)
				
				Dewdrop:AddLine(
				'icon', "Interface\\GossipFrame\\AvailableQuestIcon",
				'text', Crayon:Green(L["Left-Click"]) .. L[" on a quest title to paste the "] .. Crayon:Green("[Quest Link]") ..L[" into the chatbox."]
				)

				Dewdrop:AddLine(
				'icon', "Interface\\GossipFrame\\AvailableQuestIcon",
				'text', Crayon:Green("Control-Left-Click") .. " on a quest title to open the specific quest in the Quest Log."
				)
				
				Dewdrop:AddLine(
				'text', Crayon:Red("(You must have the daily quest in your Quest Log to use this feature.)")
				)				
				
				if (CARTOGRAPH_LOADED == true) or (TOMTOM_LOADED == true) or (CARTOGRAPH_THREE_LOADED == true) then
					Dewdrop:AddLine(
					'icon', "Interface\\GossipFrame\\AvailableQuestIcon",
					'text', Crayon:Green(L["Shift-Left-Click"]) .. " on a quest title to show the waypoint."
					)
				end					
			end			

			if value == "Waypoints" then
				if (CARTOGRAPH_LOADED == true) then
					Dewdrop:AddLine(
					'text', "Cartographer Waypoints",
					'checked', self.db.char.CART_WAYPOINTS,
					'func', function ()
						self.db.char.CART_WAYPOINTS = not self.db.char.CART_WAYPOINTS						
					end,
					'isRadio', true,
					'tooltipTitle', "Cartographer Waypoints",
					'tooltipText', "Module which shows an arrow to direct you to a specified note or location."
					)	
				end
				
				if (CARTOGRAPH_THREE_LOADED == true) then
					Dewdrop:AddLine(
					'text', "Cartographer 3 Waypoints",
					'checked', self.db.char.CART_WAYPOINTS_THREE,
					'func', function ()
						self.db.char.CART_WAYPOINTS_THREE = not self.db.char.CART_WAYPOINTS_THREE						
					end,
					'isRadio', true,
					'tooltipTitle', "Cartographer 3 Waypoints",
					'tooltipText', "Module which shows an arrow to direct you to a specified note or location."
					)	
				end
				
				if (TOMTOM_LOADED == true) then
					Dewdrop:AddLine(
					'text', "Tom Tom",
					'checked', self.db.char.TOMTOM_WAYPOINTS,
					'func', function ()
						self.db.char.TOMTOM_WAYPOINTS = not self.db.char.TOMTOM_WAYPOINTS				
					end,
					'isRadio', true,
					'tooltipTitle', "Tom Tom",
					'tooltipText', "Acts as your portable navigation assistant."
					)
				end
			end
			
			if value == "MoneyRewards" then
				Dewdrop:AddLine(
					'text', "Gold Amount Filter",
					'hasArrow', true,
					'hasSlider', true,
					'sliderMin', 1,
					'sliderMinText', Crayon:Gold("1 ") .. format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\GoldIcon.tga", 12, 12, 0),
					'sliderMax', 25,
					'sliderMaxText', Crayon:Gold("25 ") .. format(TEXTURE_LINK_FORMAT, "Interface\\Addons\\DailiesQuestTracker\\Icons\\GoldIcon.tga", 12, 12, 0),
					'sliderStep', 1,
					'sliderValue', self.db.char.GoldDisplay,
					'sliderFunc', function(value)
						self.db.char.GoldDisplay = value
					end,
					'icon', "Interface\\Icons\\INV_Misc_Coin_02",
					'tooltipTitle', "Gold Amount Filter",
					'tooltipText', "Displays the minimum gold value you'd like displayed from quests."
				)
				
				Dewdrop:AddLine(
					'text', " "
				)
					
				for QuestName, GroupText in goldByKeys (DailyQuestsList) do
					if DailyQuestsList[QuestName]["area"] == nil then
						Dailies:Debug(QuestName)
					end

					if Dailies.db.char.DailyQuestsObjectives[QuestName] then
						QuestStatus = Crayon:Yellow(QuestName)					
					elseif Dailies.db.char.DailyQuestsCompleted[QuestName] then
						QuestStatus = Crayon:White(QuestName) 
					elseif Dailies.db.char.DailyQuestsAccepted[QuestName] then
						QuestStatus = "|cff2ba1ff" .. QuestName .. "|r"
					else
						QuestStatus = "|cff808080" .. QuestName .. "|r" 
					end
					
					if DailyQuestsList[QuestName]["group"] == "HInstance" then
						QuestLocation = Crayon:White(L["Heroic Instance: "])
					elseif DailyQuestsList[QuestName]["group"] == "DInstance" then
						QuestLocation = Crayon:White(L["Dungeon Instance: "])
					elseif DailyQuestsList[QuestName]["group"] == "BGPvP" then
						QuestLocation = Crayon:White(L["Battleground: "])
					else
						QuestLocation = Crayon:White("Zone: ")
					end
						
					if DailyQuestsList[QuestName]["gold"] and (self.db.char.Filters[DailyQuestsList[QuestName]["group"]] == false) then
						if	self.db.char[DailyQuestsList[QuestName].XPac] and (ExpansionGroups[DailyQuestsList[QuestName].XPac].expansionLevel <= GetAccountExpansionLevel()) then
							local LeftCoord, RightCoord = (":"):split(DailyQuestsList[QuestName]["coords"]); local LeftCoord = tonumber(ceil(LeftCoord)); local RightCoord = tonumber(ceil(RightCoord))
							if (DailyQuestsList[QuestName]["gold"] >= (self.db.char.GoldDisplay * 10000)) then
								MoneyValue = Dailies:FormatTheGold(DailyQuestsList[QuestName]["gold"], 0)
								MoneyValueTooltip = Dailies:FormatTheGold(DailyQuestsList[QuestName]["gold"], 0)						
								if (Dailies.db.char.QuestGiver == true) then
									if (LeftCoord == 0) and (RightCoord == 0) then
										Dewdrop:AddLine(
											'text', Crayon:Gold("[") .. MoneyValue .. Crayon:Gold("] ") .. QuestStatus,
											'icon', DailyQuestsList[QuestName]["icon"],
											'iconHeight', 14,
											'iconWidth', 14,
											'tooltipTitle', format(TEXTURE_LINK_FORMAT, DailyQuestsList[QuestName]["icon"], 14, 14, 0) .. " " .. QuestStatus,
											'tooltipText', QuestLocation .. DailyQuestsList[QuestName]["area"] .. "\n" .. Crayon:White(L["Monetary Value: "]) .. MoneyValueTooltip .. "\n\n" .. Crayon:White("Quest Giver: ") .. DailyQuestsList[QuestName].qgiver .. "\n" .. Crayon:White("Location: ") .. DailyQuestsList[QuestName].qzone
										)
									else
										Dewdrop:AddLine(
											'text', Crayon:Gold("[") .. MoneyValue .. Crayon:Gold("] ") .. QuestStatus,
											'icon', DailyQuestsList[QuestName]["icon"],
											'iconHeight', 14,
											'iconWidth', 14,										
											'tooltipTitle', format(TEXTURE_LINK_FORMAT, DailyQuestsList[QuestName]["icon"], 14, 14, 0) .. " " .. QuestStatus,
											'tooltipText', QuestLocation .. DailyQuestsList[QuestName]["area"] .. "\n" .. Crayon:White(L["Monetary Value: "]) .. MoneyValueTooltip .. "\n\n" .. Crayon:White("Quest Giver: ") .. DailyQuestsList[QuestName].qgiver .. "\n" .. Crayon:White("Location: ") .. DailyQuestsList[QuestName].qszone .. Crayon:White(", ") .. DailyQuestsList[QuestName].qzone .. "\n" .. Crayon:White("Coordinates: ") .. Crayon:Green(LeftCoord)  .. Crayon:White(", ") .. Crayon:Green(RightCoord)
										)							
									end
								else
									Dewdrop:AddLine(
										'text', Crayon:Gold("[") .. MoneyValue .. Crayon:Gold("] ") .. QuestStatus,
										'icon', DailyQuestsList[QuestName]["icon"],
										'iconHeight', 14,
										'iconWidth', 14,									
										'tooltipTitle', format(TEXTURE_LINK_FORMAT, DailyQuestsList[QuestName]["icon"], 14, 14, 0) .. " " .. QuestStatus,
										'tooltipText', QuestLocation .. DailyQuestsList[QuestName]["area"] .. "\n" .. Crayon:White(L["Monetary Value: "]) .. MoneyValueTooltip
									)
								end
							end
						end
					end
				end
			end		
		end
	end
end -- function OnMenuRequest

function Dailies:ShowFractionsFrames(this)
end -- function ShowFrame